import {Component, Input, OnInit} from '@angular/core';
import {animate, style, transition, trigger} from '@angular/animations';
import {Activity, Observation, ObservationType} from '@nspop/gm-web-facade-api';
import {ActivitiesService} from '../activities.service';
import {DataCardService} from '../../api/data-card.service';

@Component({
  selector: 'app-activity-card',
  templateUrl: './activity-card.component.html',
  styleUrls: ['./activity-card.component.scss'],
  animations: [
    trigger(
      'inOutAnimation',
      [
        transition(
          ':enter',
          [
            style({ opacity: 0 }),
            animate('0.5s ease-out',
              style({opacity: 1 }))
          ]
        ),
        transition(
          ':leave',
          [
            style({ opacity: 1 })
          ]
        )
      ]
    )
  ]
})
export class ActivityCardComponent implements OnInit {
  @Input() public activity: Activity | undefined;
  public isExpanded = false;
  public ObservationType = ObservationType; // Exposed here so we can use actual enum in the html template
  constructor(
    private activitiesService: ActivitiesService,
    public dataCardService: DataCardService,
  ) { }

  ngOnInit(): void {
  }

  public getFeti(): (string | undefined)[] {
    const fetusNames = this.activity?.observations?.map(observation => observation.fetus);
    if (fetusNames) {
      return fetusNames
        .filter(name => name && name.length > 0)
        .filter((v, i, a) => a.indexOf(v) === i);
    } else {
      return [];
    }
  }

  public getMeasurement(type: ObservationType): Observation|undefined {
    return this.activity?.observations
      ?.filter(x => x.type === type)
      .sort((a, b) => b.dateOfMeasurement.getTime() - a.dateOfMeasurement.getTime())?.[0];
  }

  public getSingleMeasurementValue(type: ObservationType): string {
    const measurement = this.getMeasurement(type)?.values?.[0];
    if (!measurement) {
      return '';
    }
    return `${measurement.value || ''} ${measurement.unit || ''}`;
  }

  public getBloodPressure(): string {
    const observation = this.getMeasurement(this.ObservationType.BloodPressure);
    if (!observation) {
      return '';
    }
    return `${observation.values[0]?.value || ''} / ${observation.values[1]?.value || ''} ${observation.values[0]?.unit || ''}`;
  }

  getActivityStatusColor(status: string): string {
    if (status === 'planned') {
      return '#4228C4';
    } else if (status === 'expected') {
      return '#89C5CC';
    } else if (status === 'finished') {
      return '#646383';
    } else { return '#FFF'; }
  }

  editActivity(): void {
    if (!this.activity) { return; }
    this.activitiesService.emitOpenActivityModalEvent(this.activity);
  }

  public async cancelActivity(): Promise<void> {
    if (!this.activity) { return; }
    await this.activitiesService.cancelActivity(this.activity);
  }

  addOrEditNote(): void {
    if (!this.activity) { return; }
    this.activitiesService.emitOpenNoteModalEvent(this.activity, false);
  }

  addMeasurements(): void {
    if (!this.activity) { return; }
    this.activitiesService.emitOpenMeasurementsModalEvent(this.activity);
  }

  public hasMeasurements(): boolean {
    const observations: Array<Observation> = this.activity?.observations || [];
    return observations.length > 0;
  }
}
