import { ComponentFixture, TestBed } from '@angular/core/testing';

import {ActivityWrapper, EditPlanModalComponent} from './edit-plan-modal.component';
import {WeekNumberPipe} from '../../pipes/week-number.pipe';
import {MatButtonModule} from '@angular/material/button';
import {ModalComponent} from '../../modals/modal/modal.component';
import {Component, Input} from '@angular/core';
import {MatIcon, MatIconModule} from '@angular/material/icon';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {DirtyGuard} from '../../confirm-navigation-if-dirty.framework';
import {ActivitiesService} from '../activities.service';
import {Subject} from 'rxjs';
import {Activity, CarePlan} from '@nspop/gm-web-facade-api';
import {CarePlanService} from '../../api/care-plan.service';

@Component({
  // tslint:disable-next-line:component-selector
  selector: 'mat-icon',
  template: '<span></span>'
})
class MockMatIconComponent {
  @Input() svgIcon: any;
  @Input() fontSet: any;
  @Input() fontIcon: any;
}

describe('EditPlanModalComponent', () => {
  let component: EditPlanModalComponent;
  let fixture: ComponentFixture<EditPlanModalComponent>;

  let activitiesServiceStub: ActivitiesService;
  let carePlanSubjectStub: Subject<CarePlan>;
  let carePlanServiceStub: CarePlanService;

  beforeEach(async () => {
    activitiesServiceStub = {
      carePlan: {
        id: '123',
        version: 456,
        activities: [
          {id: '0', status: 'finished', ownership: 'currentUser'},
          {id: '1', status: 'planned', ownership: 'otherUser'},
          {id: '2', status: 'expected', ownership: 'currentUser'},
          {id: '3', status: 'planned', ownership: 'currentUser'},
          {id: '4', status: 'cancelled', ownership: 'currentUser'},
        ]
      } as any as CarePlan,
    } as any;

    carePlanSubjectStub = new Subject<CarePlan>();

    carePlanServiceStub = {
      carePlan$: carePlanSubjectStub.asObservable(),
      update: () => carePlanSubjectStub.asObservable(),
      refresh: () => {
      },
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ EditPlanModalComponent, WeekNumberPipe, ModalComponent, WeekNumberPipe],
      imports: [MatIconModule, MatButtonModule, ReactiveFormsModule, FormsModule, MatCheckboxModule],
      providers: [
        {provide: DirtyGuard, useValue: {canDeactivate: () => true}},
        {provide: ActivitiesService, useValue: activitiesServiceStub},
        {provide: CarePlanService, useValue: carePlanServiceStub},
      ]
    })
    .overrideModule(MatIconModule, {
      remove: {
        declarations: [MatIcon],
        exports: [MatIcon]
      },
      add: {
        declarations: [MockMatIconComponent],
        exports: [MockMatIconComponent]
      }
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(EditPlanModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
    component.carePlan = activitiesServiceStub.carePlan;
    component.wrapCarePlanActivities();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should return true if some but not all activities are selected', () => {
    setAllActivitiesSelection(false);
    expect(component.someButNotAllEditableSelected()).toBeFalse();
    component.editableActivities[0].selected = true;
    expect(component.someButNotAllEditableSelected()).toBeTrue();
  });

  it('should return false if all activities are selected', () => {
    setAllActivitiesSelection(true);
    expect(component.someButNotAllEditableSelected()).toBeFalse();
  });

  it('should only apply to editable activities', () => {
    // Setting editable to selected, and uneditable to unselected.
    component.activities.forEach((a) => a.selected = a.editable);
    expect(component.allEditableSelected()).toBe(true);
    expect(component.someButNotAllEditableSelected()).toBe(false);


    // Setting just 1 editable to selected. Rest unselected
    component.editableActivities[0].selected = false;
    expect(component.allEditableSelected()).toBe(false);
    expect(component.someButNotAllEditableSelected()).toBe(true);

    // Setting editable to unselected, and uneditable to selected.
    component.activities.forEach((a) => a.selected = !a.editable);
    expect(component.allEditableSelected()).toBe(false);
    expect(component.someButNotAllEditableSelected()).toBe(false);
  });

  it('should toggle all editable activities', () => {
    setAllActivitiesSelection(false);
    expect(component.allEditableSelected()).toBe(false);
    component.toggleAll();
    expect(component.allEditableSelected()).toBe(true);
    component.toggleAll();
    expect(component.allEditableSelected()).toBe(false);
  });

  it('should set all to true if only some editable activities are selected', () => {
    setAllActivitiesSelection(false);
    component.editableActivities[0].selected = true;
    expect(component.allEditableSelected()).toBe(false);
    component.toggleAll();
    expect(component.allEditableSelected()).toBe(true);
  });

  it('toggles only editable activities', () => {
    setAllActivitiesSelection(false);
    component.toggleAll();
    expect(getUneditableActivities()[0].selected).toEqual(false);
    component.toggleAll();
    expect(getUneditableActivities()[0].selected).toEqual(false);
  });

  it('filters editableActivities, so that it contains only activities who are not finished or from other users, when set visible', () => {
    component.carePlan = undefined;
    component.wrapCarePlanActivities();
    expect(component.editableActivities).toEqual([]);

    component.setVisible(true);
    component.editableActivities.forEach((a) => {
      expect(a.activity.status).not.toBe('finished');
      expect(a.activity.ownership).not.toBe('otherUser');
    });
    expect(component.editableActivities.length).not.toBe(0);
  });

  it('sets editable acitivities with status = cancelled to false, true otherwise', () => {
    component.carePlan = undefined;
    component.wrapCarePlanActivities();
    expect(component.selectedEditableActivities).toEqual([]);

    component.setVisible(true);
    component.selectedEditableActivities.forEach((a) => {
      if (a.activity.status === 'cancelled') {
        expect(a.selected).toBe(false);
      } else {
        expect(a.selected).toBe(true);
      }
    });
    expect(component.selectedEditableActivities.length).not.toBe(0);
  });

  it('closes modal if submitting carePlan is a success', () => {
    component.setVisible(true);
    component.carePlan = {
      id: '123',
      version: 456,
      activities: [ {status: 'expected', id: '1', ownership: 'currentUser'} ]
    } as any as CarePlan;
    component.wrapCarePlanActivities();

    component.submit();
    carePlanSubjectStub.next();
    expect(component.isVisible()).toBe(false);
  });

  it('does not submit if there is no careplan set', () => {
    spyOn(carePlanServiceStub, 'update');
    component.setVisible(true);
    component.carePlan = undefined;
    component.wrapCarePlanActivities();

    component.submit();
    expect(carePlanServiceStub.update).toHaveBeenCalledTimes(0);
  });

  describe('ActivityWrapper', () => {
    it('sets activity.status = cancelled if not selected and allowed to change', () => {
      const wrapper = new ActivityWrapper({status: 'expected', ownership: 'currentUser'} as Activity);
      expect(wrapper.selected).toBe(true);

      wrapper.selected = false;
      expect(wrapper.activity.status).toBe('cancelled');
    });

    it('sets activity.status = expected if selected, allowed to change, and was cancelled before', () => {
      const wrapper = new ActivityWrapper({status: 'cancelled', ownership: 'currentUser'} as Activity);
      expect(wrapper.selected).toBe(false);

      wrapper.selected = true;
      expect(wrapper.activity.status).toBe('expected');
    });

    it('does not change activity.status if activity is selected and was not cancelled before', () => {
      const wrapper = new ActivityWrapper({status: 'expected', ownership: 'currentUser'} as Activity);
      expect(wrapper.selected).toBe(true);

      wrapper.selected = true;
      expect(wrapper.activity.status).toBe('expected');
    });

    it('changes back to default when toggled back and forth', () => {
      const wrapper = new ActivityWrapper({status: 'planned', ownership: 'currentUser'} as Activity);
      expect(wrapper.selected).toBe(true);
      wrapper.selected = false;
      expect(wrapper.activity.status).toBe('cancelled');
      wrapper.selected = true;
      expect(wrapper.activity.status).toBe('planned');
    });
  });

  function setAllActivitiesSelection(value: boolean): void {
    component.activities.forEach((a) => a.selected = value);
  }

  function getUneditableActivities(): ActivityWrapper[] {
    return component.activities.filter((a) => !a.editable);
  }

});
