/* tslint:disable:max-line-length */
import {ComponentFixture, fakeAsync, flush, TestBed} from '@angular/core/testing';
import {AppComponent} from './app.component';
import {MatDialog} from '@angular/material/dialog';
import {BehaviorSubject, of} from 'rxjs';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {UserContextService} from './services/user-context.service';
import {NamePipe} from './patient-information/name.pipe';
import {CprPipe} from './pipes/cpr.pipe';
import {AppModule} from './app.module';
import {PatientContextService} from './services/patient-context.service';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {DataCard} from '@nspop/gm-web-facade-api';
import {WindowService} from './services/window.service';
import {DialogService} from './modals/dialog/dialog.service';
import {RoleSelectorComponent} from './role-selector/role-selector.component';
import {CprSearchComponent} from './cpr-search/cpr-search.component';
import {PartialDataCardError} from './services/error.service';
import {DataCardService} from './api/data-card.service';
import {CarePlanService} from './api/care-plan.service';
import {ObservationService} from './api/observation.service';
import {SessionStorageService} from './services/session-storage.service';
import {ChangeDetectorRef} from '@angular/core';
import {RouterTestingModule} from '@angular/router/testing';
import {createPatientContextSpyObject} from './services/patient-context.service.spec';
import {getPropertyAsSpy} from '../utils/spy';

describe('AppComponent', () => {
  let component: AppComponent;
  let fixture: ComponentFixture<AppComponent>;
  let documentStatusStub: string;
  let dataCardStub: DataCard;
  let dataCardResponsesSubjectStub: BehaviorSubject<DataCard[]>;

  let dataCardService: DataCardService;
  let carePlanService: CarePlanService;
  let observationService: ObservationService;

  let userContextServiceStub: UserContextService;
  let windowServiceStub: WindowService;
  let sessionStorageService: SessionStorageService;
  let patientContextStub: jasmine.SpyObj<PatientContextService>;
  let dialogServiceStub: DialogService;

  let styleStub: CSSStyleDeclaration;
  let sidebarStub: any;

  let cprSearchComponentStub: CprSearchComponent;

  let changeDetectorRefStub: ChangeDetectorRef;

  beforeEach(async () => {
    documentStatusStub = 'ACTIVE';
    dataCardStub = {
      identifier: '1',
      episodeOfCareIdentifier: 'eoc1234',
      documentStatus: documentStatusStub,
      languageSection: {
        translationRequirements: {}
      },
      recordTarget: {
        identifier: '1008821234',
        name: {
          firstNames: ['Peter'],
          middleNames: [],
          lastName: 'Parker'
        }
      },
      personalDataSection: {
        cpr: '1008821234',
        name: {
          firstNames: ['Peter'],
          middleNames: [],
          lastName: 'Parker'
        },
        address: {}
      },
      socialHistorySection: {
        partner: {
          translationLanguage: 'something',
          nationality: 'whetever',
          needsTranslation: 'yes'
        }
      },
      expectedDateOfDeliverySection: {
        expectedDateOfDeliveryFromLastPeriod: {}
      }
    } as any;

    dataCardResponsesSubjectStub = new BehaviorSubject<DataCard[]>([dataCardStub]);

    dataCardService = {
      allDataCards$: dataCardResponsesSubjectStub.asObservable(),
      selectedDataCard$: of(undefined),
      isLoading$: of(false),
      refresh: () => {
      },
      refreshAllDataCards: () => {
      },
      refreshSelectedDataCard: () => {
      }
    } as any;

    carePlanService = {
      refresh: () => {
      },
    } as any;

    observationService = {
      refresh: () => {
      },
    } as any;

    userContextServiceStub = {
      selectedRole$: of({authorizationCode: 'LK34Q', educationCode: '9495', educationType: 'Bandagist'}),
      userContext$: of({
        commonName: 'Lars Larsen', organizationName: 'Company A/S', eligibleRoles: [
          {authorizationCode: 'J0184', educationCode: '7170', educationType: 'Læge'},
          {authorizationCode: 'LK34Q', educationCode: '9495', educationType: 'Bandagist'}
        ]
      }),
      eligibleRoles$: of({authorizationCode: 'LK34Q', educationCode: '9495', educationType: 'Bandagist'}),
      isContextInitialized: () => true,
      isRoleAndOrgSelected: () => true,
      selectedOrganizationSor$: of('sorId'),
      keepAlive$: of({configuredIdleTimeout: 400}),
      keepAlive: () => {
      }
    } as any;

    windowServiceStub = {
      nativeWindow: () => {
      },
      getXYScroll: () => {
      },
    } as any;

    sessionStorageService = {
      getPatientContext: () => {
      },
      setPatientContext: () => {
      },
      setPatientContextCpr: () => {
      },
      setPatientContextEpisodeOfCare: () => {
      },
      setPatientContextXBreakTheGlassReason: () => {
      },
      deletePatientContext: () => {
      },
    } as any;

    patientContextStub = createPatientContextSpyObject();

    dialogServiceStub = {
      show: () => {
      },
    } as any;

    styleStub = {
      top: '69px',
      left: '69px',
      position: 'auto',
    } as any;

    sidebarStub = {
      nativeElement: {
        style: styleStub,
      } as any,
    } as any;

    cprSearchComponentStub = {
      focus: () => {
      },
    } as any;

    changeDetectorRefStub = {
      detectChanges: () => {
      }
    } as any;

    await TestBed.configureTestingModule({
      declarations: [AppComponent, NamePipe, CprPipe],
      imports: [
        HttpClientTestingModule,
        MatIconTestingModule,
        AppModule,
        RouterTestingModule,
      ],
      providers: [
        {provide: DataCardService, useValue: dataCardService},
        {provide: CarePlanService, useValue: carePlanService},
        {provide: ObservationService, useValue: observationService},
        {provide: MatDialog, useValue: {}},
        {provide: UserContextService, useValue: userContextServiceStub},
        {provide: WindowService, useValue: windowServiceStub},
        {provide: PatientContextService, useValue: patientContextStub},
        {provide: DialogService, useValue: dialogServiceStub},
        {provide: ChangeDetectorRef, useValue: changeDetectorRefStub},
      ]
    })
      .compileComponents();
    fixture = TestBed.createComponent(AppComponent);
    component = fixture.componentInstance;

    // tslint:disable-next-line
    spyOn(component['router'], 'navigate');
  });

  it('should create the app', () => {
    expect(component).toBeTruthy();
  });

  it(`shows correct title`, () => {
    expect(component.title).toEqual('Graviditetsportalen');
  });

  describe('when cpr is set', () => {
    beforeEach(() => {
      const cpr = 'cpr1234';
      patientContextStub.getCurrentCpr.and.returnValue(cpr);
      getPropertyAsSpy(patientContextStub, 'cpr$').and.returnValue(of(cpr));
    });

    describe('and patient exists', () => {
      beforeEach(() => {
        const patientExists = true;
        patientContextStub.getCurrentPatientExists.and.returnValue(patientExists);
        getPropertyAsSpy(patientContextStub, 'patientExists$').and.returnValue(of(patientExists));
      });

      describe('and episodeOfCare is set', () => {
        beforeEach(() => {
          const episodeOfCare = dataCardStub.episodeOfCareIdentifier;
          patientContextStub.getCurrentEpisodeOfCare.and.returnValue(episodeOfCare);
          getPropertyAsSpy(patientContextStub, 'episodeOfCare$').and.returnValue(of(episodeOfCare));
          patientContextStub.isAllowedToAccessData.and.returnValue(true);
        });

        describe('and patient has privately marked data', () => {
          beforeEach(() => {
            const hasPrivatelyMarkedData = true;
            patientContextStub.getCurrentHasPrivatelyMarkedData.and.returnValue(hasPrivatelyMarkedData);
            getPropertyAsSpy(patientContextStub, 'hasPrivatelyMarkedData$').and.returnValue(of(hasPrivatelyMarkedData));
            patientContextStub.isAllowedToAccessData.and.returnValue(false);
          });

          describe('and xBreakTheGlassReason is set', () => {
            beforeEach(() => {
              const xBreakTheGlassReason = 'Værdispring';
              patientContextStub.getCurrentXBreakTheGlassReason.and.returnValue(xBreakTheGlassReason);
              getPropertyAsSpy(patientContextStub, 'xBreakTheGlassReason$').and.returnValue(of(xBreakTheGlassReason));
              patientContextStub.isAllowedToAccessData.and.returnValue(true);
            });

            it('should render patientHeaderColumn tolkebistand', fakeAsync(() => {
              fixture.detectChanges();
              const compiled = fixture.nativeElement;
              expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance').textContent).toBeTruthy();
            }));
          }); // and xBreakTheGlassReason is set END

          it('should not render patientHeaderColumn tolkebistand', fakeAsync(() => {
            fixture.detectChanges();
            const compiled = fixture.nativeElement;
            expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance')).toBe(null);
          }));
        }); // and patient has privately marked data END

        describe('and data is not privately marked', () => {
          beforeEach(() => {
            const hasPrivatelyMarkedData = undefined;
            patientContextStub.getCurrentHasPrivatelyMarkedData.and.returnValue(hasPrivatelyMarkedData);
            getPropertyAsSpy(patientContextStub, 'hasPrivatelyMarkedData$').and.returnValue(of(hasPrivatelyMarkedData));
          });

          it('should render patientHeaderColumn tolkebistand', fakeAsync(() => {
            fixture.detectChanges();
            const compiled = fixture.nativeElement;
            expect(compiled.querySelector('.patientContext > .content > .patientInterpretingAssistance').textContent).toBeTruthy();
          }));
        }); // and data is not privately marked END

        describe('and most relevant dataCard is partial', () => {
          beforeEach(() => {
            dataCardStub.documentStatus = 'PARTIAL';
          });

          describe('and consent is NOT given', () => {
            beforeEach(() => {
              const xBreakTheGlassReason = undefined;
              patientContextStub.getCurrentXBreakTheGlassReason.and.returnValue(xBreakTheGlassReason);
              getPropertyAsSpy(patientContextStub, 'xBreakTheGlassReason$').and.returnValue(of(xBreakTheGlassReason));
            });

            it('throws a PartialDataCardError', fakeAsync(() => {
              fixture.detectChanges();
              expect(() => flush()).toThrow(new PartialDataCardError());
              expect(() => flush()).toThrow(new PartialDataCardError()); // Has to call x2
            }));
          });

          describe('and consent is given', () => {
            beforeEach(() => {
              const xBreakTheGlassReason = 'Consent';
              patientContextStub.getCurrentXBreakTheGlassReason.and.returnValue(xBreakTheGlassReason);
              getPropertyAsSpy(patientContextStub, 'xBreakTheGlassReason$').and.returnValue(of(xBreakTheGlassReason));
            });

            it('does not throw a PartialDataCardError', fakeAsync(() => {
              fixture.detectChanges();
              expect(() => flush()).not.toThrow(new PartialDataCardError());
            }));
          });
        });

        it('should render patientHeaderColumn name', fakeAsync(() => {
          fixture.detectChanges();
          const compiled = fixture.nativeElement;
          expect(compiled.querySelector('.patientContext > .content > .patientName').textContent).toContain('Peter Parker');
        }));
      }); // and episodeOfCare is set END

      it('sets episode of care if there is an active datacard', fakeAsync(() => {
        fixture.detectChanges();
        expect(patientContextStub.setEpisodeOfCare).toHaveBeenCalled();
        expect(patientContextStub.setEpisodeOfCare).toHaveBeenCalledWith(dataCardStub.episodeOfCareIdentifier);
      }));

      describe('and episode of care is not set/undefined', () => {
        beforeEach(() => {
          const episodeOfCare = undefined;
          patientContextStub.getCurrentEpisodeOfCare.and.returnValue(episodeOfCare);
          getPropertyAsSpy(patientContextStub, 'episodeOfCare$').and.returnValue(of(episodeOfCare));
        });

        it('does not refresh carePlan', () => {
          spyOn(carePlanService, 'refresh');
          fixture.detectChanges();
          expect(carePlanService.refresh).toHaveBeenCalledTimes(0);
        });

        it('does not refresh observations', () => {
          spyOn(observationService, 'refresh');
          fixture.detectChanges();
          expect(observationService.refresh).toHaveBeenCalledTimes(0);
        });
      }); // and episode of care is not set/undefined END

    }); // and patient exists END

    it('sets hasPatientInContext = true', () => {
      fixture.detectChanges();
      expect(component.hasPatientInContext).toBe(true);
    });

    it('does not set active episode of care if most relevant dataCard is NEW', () => {
      spyOn(component, 'getMostRelevantDataCard').and.returnValue({
          documentStatus: 'NEW',
          recordTarget: {identifier: 'cpr1234'}
        } as any
      );
      fixture.detectChanges();
      expect(patientContextStub.setEpisodeOfCare).toHaveBeenCalledTimes(0);
    });

    it('does not set active episode of care if most relevant dataCard is CLOSED', () => {
      spyOn(component, 'getMostRelevantDataCard').and.returnValue({
          documentStatus: 'CLOSED',
          recordTarget: {identifier: 'cpr1234'}
        } as any
      );
      fixture.detectChanges();
      expect(patientContextStub.setEpisodeOfCare).toHaveBeenCalledTimes(0);
    });

    describe('and patient does not exist', () => {
      beforeEach(() => {
        const patientExists = false;
        patientContextStub.getCurrentPatientExists.and.returnValue(patientExists);
        getPropertyAsSpy(patientContextStub, 'patientExists$').and.returnValue(of(patientExists));
      });

      it('does not refresh carePlan', () => {
        spyOn(carePlanService, 'refresh');
        fixture.detectChanges();
        expect(carePlanService.refresh).toHaveBeenCalledTimes(0);
      });

      it('does not refresh observations', () => {
        spyOn(observationService, 'refresh');
        fixture.detectChanges();
        expect(observationService.refresh).toHaveBeenCalledTimes(0);
      });
    }); // and patient does not exist END

    it('refreshes dataCards', () => {
      spyOn(dataCardService, 'refresh');
      fixture.detectChanges();
      expect(dataCardService.refresh).toHaveBeenCalledTimes(1);
    });
  }); // when cpr is set END

  describe('when cpr is not set/undefined', () => {
    beforeEach(() => {
      const cpr = undefined;
      patientContextStub.getCurrentCpr.and.returnValue(cpr);
      getPropertyAsSpy(patientContextStub, 'cpr$').and.returnValue(of(cpr));
    });

    it('sets hasPatientInContext = false', () => {
      fixture.detectChanges();
      expect(component.hasPatientInContext).toBe(false);
    });

    it('does not refresh dataCards', () => {
      spyOn(dataCardService, 'refresh');
      fixture.detectChanges();
      expect(dataCardService.refresh).toHaveBeenCalledTimes(0);
    });

    it('does not refresh carePlan', () => {
      spyOn(carePlanService, 'refresh');
      fixture.detectChanges();
      expect(carePlanService.refresh).toHaveBeenCalledTimes(0);
    });

    it('does not refresh observations', () => {
      spyOn(observationService, 'refresh');
      fixture.detectChanges();
      expect(observationService.refresh).toHaveBeenCalledTimes(0);
    });
  });

  describe('when windowService.getXYScroll returns y < 72', () => {
    beforeEach(async () => {
      spyOn(windowServiceStub, 'getXYScroll').and.returnValue({x: 0, y: 71});
    });

    it('sets style.top = 72-y px', () => {
      fixture.detectChanges();
      component.sidebar = sidebarStub;
      component.allowYScrollFrame();
      expect(styleStub.top).toBe('1px');
    });
  });

  describe('when windowService.getXYScroll returns y >= 72', () => {
    beforeEach(async () => {
      spyOn(windowServiceStub, 'getXYScroll').and.returnValue({x: 0, y: 72});
    });

    it('sets style.top = 0px', () => {
      fixture.detectChanges();
      component.sidebar = sidebarStub;
      component.allowYScrollFrame();
      expect(styleStub.top).toBe('0px');
    });
  });

  it('calls dialogService.show when openChangeRoleDialog is called', () => {
    spyOn(dialogServiceStub, 'show');
    fixture.detectChanges();
    component.openChangeRoleDialog();
    expect(dialogServiceStub.show).toHaveBeenCalledTimes(1);
    expect(dialogServiceStub.show).toHaveBeenCalledWith(RoleSelectorComponent, {width: 'fit-content'});
  });

  it('sets feedbackModalOpen = true when openFeedbackDialog is called', () => {
    component.feedbackModalOpen = false;
    fixture.detectChanges();
    component.openFeedbackDialog();
    expect(component.feedbackModalOpen).toBe(true);
  });

  describe('when getting all dataCards', () => {
    const selectedEpisodeOfCare = 'eoc1234';
    const getMockDataCards = () => {
      return [
        {documentStatus: 'NEW', documentUniqueId: 'a1'},
        {documentStatus: 'CLOSED', documentUniqueId: 'b2'},
        {documentStatus: 'CLOSED', documentUniqueId: 'c3'},
        {documentStatus: 'ACTIVE', documentUniqueId: 'd4'},
        {documentStatus: 'CLOSED', documentUniqueId: 'e5'},
        {documentStatus: 'CLOSED', documentUniqueId: '66'},
        {documentStatus: 'CLOSED', documentUniqueId: '77'},
        {documentStatus: 'PARTIAL', documentUniqueId: '88'},
        {documentStatus: 'CLOSED', documentUniqueId: '99'},
        {documentStatus: 'CLOSED', documentUniqueId: 'ab'},
      ] as DataCard[];
    };

    it('should return the most relevant one based on status', () => {
      let dataCards = getMockDataCards();
      patientContextStub.getCurrentEpisodeOfCare.and.returnValue(selectedEpisodeOfCare);
      fixture.detectChanges();

      let mostRelevantDataCard = component.getMostRelevantDataCard(dataCards);
      expect(mostRelevantDataCard?.documentStatus).toBe('ACTIVE');
      expect(mostRelevantDataCard?.documentUniqueId).toBe('d4');

      dataCards = dataCards.filter(card => card.documentUniqueId !== 'd4');
      mostRelevantDataCard = component.getMostRelevantDataCard(dataCards);
      expect(mostRelevantDataCard?.documentStatus).toBe('NEW');
      expect(mostRelevantDataCard?.documentUniqueId).toBe('a1');

      dataCards = dataCards.filter(card => card.documentUniqueId !== 'a1');
      mostRelevantDataCard = component.getMostRelevantDataCard(dataCards);
      expect(mostRelevantDataCard?.documentStatus).toBe('PARTIAL');
      expect(mostRelevantDataCard?.documentUniqueId).toBe('88');

      dataCards = dataCards.filter(card => card.documentUniqueId !== '88');
      mostRelevantDataCard = component.getMostRelevantDataCard(dataCards);
      expect(mostRelevantDataCard?.documentStatus).toBe('CLOSED');
      expect(mostRelevantDataCard?.documentUniqueId).toBe('b2');

      dataCards = [];
      mostRelevantDataCard = component.getMostRelevantDataCard(dataCards);
      expect(mostRelevantDataCard).toBe(undefined);
    });

    it('should give me whatever matches currentEpisodeOfCare', () => {
      const dataCards = getMockDataCards();
      const episodeOfCareIdentifier = '1337abcd';
      dataCards.push(
        {
          documentStatus: 'CLOSED',
          documentUniqueId: '12345678',
          episodeOfCareIdentifier
        } as DataCard);
      patientContextStub.getCurrentEpisodeOfCare.and.returnValue(episodeOfCareIdentifier);
      fixture.detectChanges();

      const activeDatacard = component.getMostRelevantDataCard(dataCards);
      expect(activeDatacard?.documentUniqueId).toBe('12345678');
    });
  });
});
