import {ChangeDetectorRef, Component, ElementRef, HostListener, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {UserContextService} from './services/user-context.service';
import {RoleSelectorComponent} from './role-selector/role-selector.component';
import {MatIconRegistry} from '@angular/material/icon';
import {DomSanitizer} from '@angular/platform-browser';
import {DataCard} from '@nspop/gm-web-facade-api';
import {DialogService} from './modals/dialog/dialog.service';
import {PatientContextService} from './services/patient-context.service';
import {WindowService} from './services/window.service';
import {first, take, throttleTime} from 'rxjs/operators';
import {CprDoesNotExistError, PartialDataCardError} from './services/error.service';
import {DataCardService} from './api/data-card.service';
import {CarePlanService} from './api/care-plan.service';
import {ObservationService} from './api/observation.service';
import {asyncScheduler, Subject, Subscription} from 'rxjs';
import {KeepAliveConfigService} from './services/keep-alive-config.service';
import {Router} from '@angular/router';
import {appUrls} from '../utils/URL_STRING_LITERALS';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit, OnDestroy {
  title = 'Graviditetsportalen';
  dataCard: DataCard | undefined;
  allDataCards: DataCard[] = [];
  hasPatientInContext = false;
  hasEpisodeOfCareInContext = false;
  hasXBreakTheGlassReasonInContext = false;
  hasPrivatelyMarkedDataInContext = false;
  patientExists: boolean | undefined;
  feedbackModalOpen = false;
  prevEpisodeOfCareIdentifier: string | undefined = undefined;
  initializationFailed: boolean = false;
  private refreshSubject: Subject<void> = new Subject();

  @ViewChild('sidebar') sidebar?: ElementRef<HTMLElement>;
  private subscription: Subscription|undefined;

  constructor(
    public dataCardService: DataCardService,
    public carePlanService: CarePlanService,
    public observationService: ObservationService,
    public userContextService: UserContextService,
    public dialogService: DialogService,
    public patientContextService: PatientContextService,
    private matIconRegistry: MatIconRegistry,
    private domSanitizer: DomSanitizer,
    private windowService: WindowService,
    private changeDetectorRef: ChangeDetectorRef,
    private keepAliveConfigService: KeepAliveConfigService,
    private router: Router
  ) {
  }

  ngOnDestroy(): void {
    this.subscription?.unsubscribe();
  }

  public ngOnInit(): void {
    this.registerIcons();
    this.registerSmileys();
    // Make sure that userContext has loaded before loading anything else.
    // first() pipe unsubscribes after first value is received, so we don't 'over-subscribe'.
    this.userContextService.userContext$.pipe(
      first()
    ).subscribe(() => {
      // Business logic
      this.setupDataCardSubscription();
      this.setupCprSubscription();
      this.setupEpisodeOfCareSubscription();

      // Variables for UI rendering
      this.setupXBreakTheGlassReasonSubscription();
      this.setupPrivatelyMarkedSubscription();
      this.setupPatientExistsSubscription();

      this.userContextService.keepAlive$
        .pipe(take(1))
        .subscribe((keepAliveContext) =>
          this.keepAliveConfigService.keepAliveContext.next(keepAliveContext)
        );
      const THROTTLE_WAIT = 300000;
      this.subscription = this.refreshSubject
        .pipe(throttleTime(THROTTLE_WAIT, asyncScheduler, {leading: true, trailing: false}))
        .subscribe(() => {
          this.userContextService.keepAlive();
        });
      },
      () => {
        this.initializationFailed = true;
      }
    );
  }

  private registerIcons(): void {
    this.matIconRegistry.addSvgIcon('ic-add',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_add.svg'));
    this.matIconRegistry.addSvgIcon('ic-add-colored',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_add-colored.svg'));
    this.matIconRegistry.addSvgIcon('ic-calendar',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_calendar.svg'));
    this.matIconRegistry.addSvgIcon('ic-care-plan',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_fagperson-mand.svg'));
    this.matIconRegistry.addSvgIcon('ic-care-role',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_fagperson-role.svg'));
    this.matIconRegistry.addSvgIcon('ic-close',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_close-large.svg'));
    this.matIconRegistry.addSvgIcon('ic-close-small-faded',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_close-small-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-close-small-highlighted',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_close-small-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-edit',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_edit.svg'));
    this.matIconRegistry.addSvgIcon('ic-edit-highlight',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_edit-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-info',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_info.svg'));
    this.matIconRegistry.addSvgIcon('ic-lock',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_lock.svg'));
    this.matIconRegistry.addSvgIcon('ic-more',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_more.svg'));
    this.matIconRegistry.addSvgIcon('ic-overview',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_overblik.svg'));
    this.matIconRegistry.addSvgIcon('ic-pdf-highlighted',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_pdf-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-pdf-faded',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_pdf-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-search-faded',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_search-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-search-highlighted',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_search-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-trash',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_trash.svg'));
    this.matIconRegistry.addSvgIcon('ic-view',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_view-faded.svg'));
  }

  private registerSmileys(): void {
    const addSmiley = (name: string) => {
      this.matIconRegistry.addSvgIcon(`smiley-${name}`,
        this.domSanitizer.bypassSecurityTrustResourceUrl(`assets/img/smileys/smiley-${name}.svg`));
      this.matIconRegistry.addSvgIcon(`smiley-${name}@faded`,
        this.domSanitizer.bypassSecurityTrustResourceUrl(`assets/img/smileys/smiley-${name}@faded.svg`));
    };
    addSmiley('super-sad');
    addSmiley('sad');
    addSmiley('meh');
    addSmiley('happy');
    addSmiley('super-happy');
  }

  public shouldDisplayRoleSelector(): boolean {
    return !this.userContextService.isContextInitialized() && !this.userContextService.isRoleAndOrgSelected();
  }

  public shouldDisplayMainApplication(): boolean {
    return this.userContextService.isContextInitialized() && this.userContextService.isRoleAndOrgSelected();
  }

  public getMostRelevantDataCard(datacards: DataCard[]): DataCard | undefined {
    let relevantCard = datacards
      .find(card => card.episodeOfCareIdentifier === this.patientContextService.getCurrentEpisodeOfCare());
    if (relevantCard) {
      return relevantCard;
    }
    ['ACTIVE', 'NEW', 'PARTIAL', 'CLOSED'].some(status => {
      relevantCard = datacards.find(card => card.documentStatus === status);
      return relevantCard;
    });
    return relevantCard;
  }
  /**
   * Upon receiving dataCards from the BFF we update the episodeOfCare, and do specific actions depending on status.
   * @private
   */
  private setupDataCardSubscription(): void {
    this.dataCardService.allDataCards$.subscribe((dataCards: DataCard[]) => {
      this.allDataCards = dataCards;
      this.dataCard = this.getMostRelevantDataCard(dataCards);

      if (!this.dataCard) { return; }
      const { documentStatus, recordTarget } = this.dataCard;

      // 404CPR = Backend could not find 'FSK Stamkort' / Patient does not exist but call went well (200)
      const cprDoesNotExist = !recordTarget || recordTarget?.identifier === 'CPR404';
      if (cprDoesNotExist) { throw new CprDoesNotExistError(); }

      this.patientContextService.setPatientExists(true);

      // 'CLOSED' = DataCard belongs to an earlier pregnancy.
      // 'NEW' = Is always returned on top of the list if there are no 'ACTIVE' pregnancies.
      // 'NEW' indicates that the user should be led to the page where they can start a new pregnancy.
      const dataCardIsActive = documentStatus !== 'CLOSED' && documentStatus !== 'NEW';
      if (dataCardIsActive) {
        this.patientContextService.setEpisodeOfCare(this.dataCard.episodeOfCareIdentifier);
      }

      const dataCardIsNotPrivate = documentStatus !== 'PARTIAL';
      if (dataCardIsNotPrivate) { return; }

      const hasConsent = !!this.patientContextService.getCurrentXBreakTheGlassReason();
      if (hasConsent) { return; }

      throw new PartialDataCardError();
    });
  }

  /**
   * Every time a new cpr is set, we call the server to get the new datacards for that cpr.
   * This is the only way DataCards are updated automatically in the app.
   * @private
   */
  private setupCprSubscription(): void {
    this.patientContextService.cpr$.subscribe((cpr) => {
      this.hasPatientInContext = !!cpr;
      if (cpr) {
        this.dataCardService.refresh();
      }
    });
  }

  /**
   * Every time a new datacard is received, it will set the episodeOfCare, which results calling the server to get a new CarePlan.
   * This is the only way CarePlan is updated automatically in the app.
   * @private
   */
  private setupEpisodeOfCareSubscription(): void {
    this.hasEpisodeOfCareInContext = !!this.patientContextService.getCurrentEpisodeOfCare();
    this.patientContextService.episodeOfCare$.subscribe((episodeOfCareId) => {
      const isOldEpisodeOfCareId = this.dataCard?.episodeOfCareIdentifier === episodeOfCareId;
      this.hasEpisodeOfCareInContext = !!episodeOfCareId;
      this.dataCard = this.allDataCards.find(x => x.episodeOfCareIdentifier === episodeOfCareId);

      if (!this.hasEpisodeOfCareInContext) {
        return this.router.navigate([appUrls.noActivePathway]);
      }

      this.dataCardService.refreshSelectedDataCard();
      this.carePlanService.refresh();
      this.observationService.refresh();

      if (isOldEpisodeOfCareId) { return; }

      this.router.navigate([appUrls.base]);

    });
  }

  private setupXBreakTheGlassReasonSubscription(): void {
    this.hasXBreakTheGlassReasonInContext = !!this.patientContextService.getCurrentXBreakTheGlassReason();
    this.patientContextService.xBreakTheGlassReason$.subscribe((reason => {
      this.hasXBreakTheGlassReasonInContext = !!reason;
    }));
  }

  private setupPrivatelyMarkedSubscription(): void {
    this.hasPrivatelyMarkedDataInContext = !!this.patientContextService.getCurrentHasPrivatelyMarkedData();
    this.patientContextService.hasPrivatelyMarkedData$.subscribe((value => {
      this.hasPrivatelyMarkedDataInContext = !!value;
    }));
  }

  private setupPatientExistsSubscription(): void {
    this.patientExists = this.patientContextService.getCurrentPatientExists();
    this.patientContextService.patientExists$.subscribe((exists) => {
      this.patientExists = exists;
      this.router.navigate([appUrls.base]);
    });
  }

  openChangeRoleDialog(): void {
    this.dialogService.show(RoleSelectorComponent, {
      width: 'fit-content',
    });

  }

  @HostListener('window:scroll')
  public allowYScrollFrame(): void {
    const {x, y} = this.windowService.getXYScroll();
    const style = this.sidebar?.nativeElement.style;
    if (style) {
      style.left = `${-x}px`;
      style.top = `${0}px`;
      if (y <= 72) {
        style.top = `${72 - y}px`;
      }
    }
  }

  openFeedbackDialog(): void {
    this.feedbackModalOpen = true;
  }

  public closePatient(): void {
    console.log('TODO: this should close the patient');
  }

  @HostListener('window:keydown')
  public onKeyDown(): void {
    this.refreshSubject.next();
  }
  @HostListener('window:mousemove')
  public onMouseMove(): void {
    this.refreshSubject.next();
  }

  public getTranslationRequirements(): string {
    const translationRequirements = this.dataCard?.languageSection?.translationRequirements;
    if (!translationRequirements?.needsTranslation) {
      return '-';
    }

    if (translationRequirements.needsTranslation === 'false') {
      return 'Nej';
    }

    return translationRequirements?.language ? `Ja - ${translationRequirements.language}` : 'Ja';
  }
}
