import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {CheckboxComponent, CheckboxListComponent} from './checkbox-list.component';
import {Component, ViewChild} from '@angular/core';
import {By} from '@angular/platform-browser';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {FormsModule} from '@angular/forms';
import {AppModule} from '../app.module';
import {of} from 'rxjs/internal/observable/of';
import {DataCardService} from '../api/data-card.service';

@Component({
  selector: 'app-test-checkbox-list-component',
  template: '<app-checkbox-list #checkboxList name="test" label="Whatever" [disabled]="disabled" [(ngModel)]="test">' +
        '<app-checkbox label="test 1" value="first_test_element"></app-checkbox>' +
        '<app-checkbox label="test 2" value="second_test_element"></app-checkbox>' +
    '</app-checkbox-list>'
})
class TestCheckboxListComponent {
  @ViewChild('checkboxList') checkboxList?: CheckboxListComponent;
  public test: string[] = [];
  public disabled = false;
}

describe('CheckboxListComponent', () => {
  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ CheckboxListComponent, CheckboxComponent, TestCheckboxListComponent ],
      imports: [AngularMaterialModule, FormsModule, AppModule],
      providers: [
        {provide: DataCardService, useValue: {isDataCardClosed: of(false)}}
      ]
    })
    .compileComponents();
  });

  describe('overall component behaviour', () => {
    let component: CheckboxListComponent;
    let fixture: ComponentFixture<CheckboxListComponent>;

    beforeEach(() => {
      fixture = TestBed.createComponent(CheckboxListComponent);
      component = fixture.componentInstance;
      fixture.detectChanges();
    });

    it('should create', () => {
      expect(component).toBeTruthy();
    });

    it ('only updates value if new value is structurally different from old one', () => {
      const onChange = jasmine.createSpy('onChange');
      component.writeValue(['1', '2']);
      component.registerOnChange(onChange);
      component.writeValue(['1', '2']);
      expect(onChange).toHaveBeenCalledTimes(0);
      component.makeDirtyable();
      component.writeValue(['1', '2', '3']);
      expect(onChange).toHaveBeenCalledTimes(1);
    });

    it('removes element from model if checkbox is unchecked', () => {
      component.writeValue(['1', '2']);
      component.handleChange({display: 'whatever', checked: false, value: '1'});
      expect(component.value).toEqual(['2']);
    });

    it('adds element to model if checkbox is checked', () => {
      component.writeValue(['1', '2']);
      component.handleChange({display: 'whatever', checked: true, value: '3'});
      expect(component.value).toEqual(['1', '2', '3']);
    });

    it('does not register a change on the first value set', () => {
      // This is to avoid issues where values initialized from an async call will mark the form as dirty
      const mock = {
        onChange: (whatever: any) => undefined
      };
      const onChange = spyOn(mock, 'onChange');
      component.registerOnChange(onChange);
      component.writeValue(['1', '2']);
      expect(onChange).not.toHaveBeenCalled();
      component.makeDirtyable();
      component.writeValue(['1']);
      expect(onChange).toHaveBeenCalledWith(['1']);
    });

    it('becomes dirtyable when clicked', () => {
      component.checklistItems = [{display: '1', value: 1, checked: false}, {display: '2', value: 2, checked: false}];
      fixture.detectChanges();
      fixture.debugElement.query(By.css('mat-checkbox')).triggerEventHandler('click', {});
      fixture.detectChanges();
      expect((component as any).dirtyAble).toBeTrue();
    });
  });

  describe('integration with material checkbox', () => {
    let component: TestCheckboxListComponent;
    let fixture: ComponentFixture<TestCheckboxListComponent>;

    beforeEach(() => {
      fixture = TestBed.createComponent(TestCheckboxListComponent);
      component = fixture.componentInstance;
      fixture.detectChanges();
    });

    it ('creates a mat-checkbox for each app-checkbox', () => {
      fixture.detectChanges();
      const materialCheckboxes = fixture.debugElement.queryAll(By.css('mat-checkbox'));
      expect(materialCheckboxes.length).toBe(2);
    });

    it('fills in checkboxes based on ngModel value', (done) => {
      component.test = ['second_test_element'];
      fixture.detectChanges();
      fixture.whenStable().then(() => {
        fixture.detectChanges();
        return fixture.whenStable();
      }).then(() => {
        fixture.detectChanges();
        const materialCheckboxes = fixture.debugElement.queryAll(By.css('mat-checkbox'));
        expect(materialCheckboxes[0].nativeNode.classList.contains('mat-checkbox-checked')).toBeFalse();
        expect(materialCheckboxes[1].nativeNode.classList.contains('mat-checkbox-checked')).toBeTrue();
        done();
      });
    });

    it ('un-fills checkboxes when ngModel changes', (done) => {
      component.test = ['second_test_element'];
      fixture.detectChanges();
      fixture.whenStable().then(() => {
        fixture.detectChanges();
        return fixture.whenStable();
      }).then(() => {
        fixture.detectChanges();
        const materialCheckboxes = fixture.debugElement.queryAll(By.css('mat-checkbox'));
        expect(materialCheckboxes[0].nativeNode.classList.contains('mat-checkbox-checked')).toBeFalse();
        expect(materialCheckboxes[1].nativeNode.classList.contains('mat-checkbox-checked')).toBeTrue();
        component.test = [];
        fixture.detectChanges();
        return fixture.whenStable();
      }).then(() => {
        // So... ngModel updates are async and we're dealing with multiple (2) layers of ngModel here, so we need to wait for updates twice
        fixture.detectChanges();
        return fixture.whenStable();
      }).then(() => {
        fixture.detectChanges();
        const materialCheckboxes = fixture.debugElement.queryAll(By.css('mat-checkbox'));
        expect(materialCheckboxes[0].nativeNode.classList.contains('mat-checkbox-checked')).toBeFalse();
        expect(materialCheckboxes[1].nativeNode.classList.contains('mat-checkbox-checked')).toBeFalse();
        expect(component.checkboxList?.innerModel).toEqual([]);
        done();
      });
    });

    it ('disables checkboxes when disabled', fakeAsync(() => {
      component.disabled = true;
      fixture.detectChanges();
      tick();
      fixture.detectChanges();
      const materialCheckboxes = fixture.debugElement.queryAll(By.css('mat-checkbox'));
      expect(materialCheckboxes[0].nativeNode.classList.contains('mat-checkbox-disabled')).toBeTrue();
      expect(materialCheckboxes[1].nativeNode.classList.contains('mat-checkbox-disabled')).toBeTrue();
    }));
  });
});
