import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {DatePickerComponent} from './date-picker.component';
import {FormsModule} from '@angular/forms';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {SimpleChange, SimpleChanges} from '@angular/core';
import {MatDatepickerModule} from '@angular/material/datepicker';
import {MatMomentDateModule} from '@angular/material-moment-adapter';
import {MatInputModule} from '@angular/material/input';
import {JsonTransformerUtil} from '../../utils/json-transformer.util';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {By} from '@angular/platform-browser';
import {copy} from '../../utils/copy';
import {of} from 'rxjs/internal/observable/of';
import {DataCardService} from '../api/data-card.service';

describe('DatePickerComponent', () => {
  let component: DatePickerComponent;
  let fixture: ComponentFixture<DatePickerComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ DatePickerComponent],
      imports: [MatDatepickerModule, MatIconTestingModule, MatMomentDateModule, MatInputModule, FormsModule, NoopAnimationsModule],
      providers: [
        {provide: DataCardService, useValue: {isDataCardClosed: of(false)}}
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(DatePickerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('only registers changes if model has structural changes', () => {
    const onChange = jasmine.createSpy('onChange');
    const date = new Date();
    const date2 = JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(date)));

    component.writeValue(date);
    component.registerOnChange(onChange);
    component.writeValue(date2);
    expect(onChange).toHaveBeenCalledTimes(0);
    fixture.debugElement.query(By.css('input')).triggerEventHandler('focus', {});
    component.writeValue( makeDate('2020', '01', '01'));
    expect(onChange).toHaveBeenCalledTimes(1);
  });

  it('detects mouse hover events', (done) => {
    fixture.whenStable().then(() => {
      fixture.debugElement.triggerEventHandler('mouseenter', {});
      fixture.detectChanges();
      expect(component.hover).toBeTrue();

      fixture.debugElement.triggerEventHandler('mouseleave', {});
      fixture.detectChanges();
      expect(component.hover).toBeFalse();
      done();
    });
  });

  it('registers as touched on blur', (done) => {
    let touched = false;
    component.registerOnTouched(() => touched = true);
    fixture.whenStable().then(() => {
      fixture.debugElement.triggerEventHandler('blur', {});
      fixture.detectChanges();
      expect(touched).toBeTrue();
      done();
    });
  });

  it('updates inner value when ngModel is updated on the outside', () => {
    const date = makeDate('2020', '01', '01');
    const verify = copy(date);
    component.writeValue(date);
    expect(component.innerModel?.toDate().getUTCFullYear()).toEqual(verify.getUTCFullYear());
    expect(component.innerModel?.toDate().getUTCMonth()).toEqual(verify.getUTCMonth());
    expect(component.innerModel?.toDate().getUTCDate()).toEqual(verify.getUTCDate());
  });

  it('updates inner value when ngModel is updated on the outside to be undefined', () => {
    const date = undefined;
    component.writeValue(date);
    expect(component.innerModel).toBeUndefined();
  });

  it('makes ngModel undefined when set to not relevant',  fakeAsync(() => {
    const date = makeDate('2020', '01', '01');
    component.writeValue(date);
    fixture.detectChanges();
    component.notRelevant = true;
    component.ngOnChanges({notRelevant: {currentValue: true, previousValue: undefined} as SimpleChange} as SimpleChanges);
    tick();
    expect(component.value).toBeUndefined();
  }));

  it('re-fills the date when set to not relevant and back again',  fakeAsync(() => {
    const date = makeDate('2020', '01', '01');
    const verify = copy(date);
    component.writeValue(date);
    fixture.detectChanges();
    component.notRelevant = true;
    component.ngOnChanges({notRelevant: {currentValue: true, previousValue: undefined} as SimpleChange} as SimpleChanges);
    tick();
    expect(component.value).toBeUndefined();
    component.notRelevant = false;
    component.ngOnChanges({notRelevant: {currentValue: false, previousValue: true} as SimpleChange} as SimpleChanges);
    tick();
    expect(component.innerModel?.toDate().getUTCFullYear()).toEqual(verify.getUTCFullYear());
    expect(component.innerModel?.toDate().getUTCMonth()).toEqual(verify.getUTCMonth());
    expect(component.innerModel?.toDate().getUTCDate()).toEqual(verify.getUTCDate());
  }));

  it('does not register a change on the first value set', () => {
    // This is to avoid issues where values initialized from an async call will mark the form as dirty
    const mock = {
      onChange: (whatever: any) => undefined
    };
    const onChange = spyOn(mock, 'onChange');
    const date = makeDate('2020', '01', '01');
    component.registerOnChange(onChange);
    component.writeValue(new Date());
    expect(onChange).not.toHaveBeenCalled();
    fixture.debugElement.query(By.css('input')).triggerEventHandler('focus', {});
    component.writeValue(date);
    expect(onChange).toHaveBeenCalled();
  });

  it('becomes dirtyable when clicked', () => {
    fixture.debugElement.query(By.css('input')).triggerEventHandler('focus', {});
    fixture.detectChanges();
    expect((component as any).dirtyAble).toBeTrue();
  });
});

function makeDate(year: string, month: string, day: string): Date {
  return JsonTransformerUtil.transformDates(`${year}-${month}-${day}T00:00:00.00`);
}
