import {Component, ElementRef, HostListener, OnInit, ViewChild} from '@angular/core';
import {ChronicDiseasesSection, DataCard, OverallAssessmentSectionAllOfPrenatalCareAssessment, PosNegEnum} from '@nspop/gm-web-facade-api';
import {DataCardService} from '../api/data-card.service';
import {SwitchEncounterComponent} from '../switch-encounter/switch-encounter.component';
import {MatDialog} from '@angular/material/dialog';
import {PatientContextService} from '../services/patient-context.service';
import {environment} from '../../environments/environment';
import {WindowService} from '../services/window.service';
import ConditionsEnum = ChronicDiseasesSection.ConditionsEnum;
import CareLevelEnum = OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum;

@Component({
    selector: 'app-general-information-bar',
    templateUrl: './general-information-bar.component.html',
    styleUrls: ['./general-information-bar.component.scss']
})
export class GeneralInformationBarComponent implements OnInit {
    @ViewChild('headerContainer') public headerContainer: ElementRef | undefined;
    public headerInScrollMode = false;
    public chronicDiseases: string[] = ['-'];
    public careLevel = '?';
    public rhesusType = '?';
    public dataCard: DataCard | undefined;
    public activeEncounterExists = false;

    constructor(
        public dataCardService: DataCardService,
        private dialog: MatDialog,
        private patientContextService: PatientContextService,
        private windowService: WindowService
    ) {
    }

    ngOnInit(): void {
        this.dataCardService.selectedDataCard$
            .subscribe((dataCard: DataCard | undefined) => {
                if (dataCard) {
                    this.dataCard = dataCard;
                }
                this.chronicDiseases = this.computeSpecialConditions(
                    this.dataCard?.chronicDiseasesSection
                    || {dirty: false} as ChronicDiseasesSection); // TODO: Is missing modifiedTime and author!!
                this.careLevel = this.computeCareLevel(this.dataCard?.overallAssessmentSection?.prenatalCareAssessment?.careLevel);
                this.rhesusType = this.computeRhesusType(this.dataCard?.screeningsSection?.maternalRhD?.rhesusType);
            });

        this.dataCardService.allDataCards$.subscribe((dataCards) => {
            this.activeEncounterExists = dataCards.some(dataCard => dataCard.documentStatus === 'ACTIVE');
        });
    }

    get expectedDateOfBirth(): { route: string, params: any } {
        if (this.dataCard?.ultrasoundSection?.expectedDateOfDeliveryFromPrenatalUltrasoundScan) {
            return {
                route: '/undersøgelser-og-målinger',
                params: {tab: 1, highlight: 'birth-date-ultrasound-birth-date'}
            };
        }
        return {
            route: '/svangerskab',
            params: {tab: 1, highlight: 'terminsberegning'}
        };
    }

    public createEncounter(): void {
        this.dataCardService.create().subscribe(() => {
            this.dataCardService.refresh().then(() => {
                this.windowService.nativeWindow().location.assign(environment.base_path);
            });
        });
    }

    public swapEncounter(): void {
        const dialogRef = this.dialog.open(SwitchEncounterComponent);
        dialogRef.afterClosed().subscribe((result: DataCard) => {
            if (result) {
                this.patientContextService.setEpisodeOfCare(result.episodeOfCareIdentifier);
            }
        });
    }

    public finishEncounter(): void {
        this.dataCardService.close().subscribe(() => {
            this.windowService.nativeWindow().location.assign(environment.base_path);
        });
    }

    public computeCareLevel(carelevel?: CareLevelEnum): string {
        switch (carelevel) {
            case CareLevelEnum.Basic:
                return '1';
            case CareLevelEnum.Extended:
                return '2';
            case CareLevelEnum.SpecialistMultidisciplinaryTeam:
                return '3';
            case CareLevelEnum.HighRiskPregnancyCare:
                return '4';
            default:
                return '?';
        }
    }

    public computeRhesusType(type?: PosNegEnum): string {
        switch (type) {
            case 'positive':
                return 'Positiv';
            case 'negative':
                return 'Negativ';
            default:
                return '?';
        }
    }

    private computeSpecialConditions(chronicDiseasesSection: ChronicDiseasesSection): string[] {
        if (!chronicDiseasesSection.conditions || chronicDiseasesSection.conditions.length === 0) {
            return ['-'];
        }
        return this.mapSpecialConditions(chronicDiseasesSection.conditions);
    }

    private mapSpecialConditions(conditions: ChronicDiseasesSection.ConditionsEnum[]): string[] {
        return conditions.map(
            key => {
                switch (key) {
                    case ConditionsEnum.CirculatorySystem:
                        return 'Kredsløb';
                    case ConditionsEnum.RespiratorySystem:
                        return 'Luftveje';
                    case ConditionsEnum.Thyroid:
                        return 'Thyroidea';
                    case ConditionsEnum.Diabetes:
                        return 'Diabetes';
                    case ConditionsEnum.Epilepsy:
                        return 'Epilepsi';
                    case ConditionsEnum.MentalIllness:
                        return 'Psykisk sygdom';
                    case ConditionsEnum.Herpes:
                        return 'Herpes genitalis';
                    case ConditionsEnum.Uvi:
                        return 'Recidiverende UVI';
                    case ConditionsEnum.ConeSection:
                        return 'Keglesnit';
                    case ConditionsEnum.Mrsa:
                        return 'MRSA';
                    case ConditionsEnum.Gbs:
                        return 'Gbs';
                    default:
                        return '';
                }
            }
        );
    }

    @HostListener('window:scroll', ['$event']) // for window scroll events
    public onScroll(): void {
        if (this.headerContainer) {
            const elemTop = this.headerContainer.nativeElement.getBoundingClientRect().top;
            this.headerInScrollMode = elemTop <= 0;
        }
    }

    public getExpectedDateOfDelivery(): Date | undefined {
        return this.dataCard?.ultrasoundSection?.expectedDateOfDeliveryFromPrenatalUltrasoundScan ||
            this.dataCard?.expectedDateOfDeliverySection?.expectedDateOfDeliveryFromLastPeriod?.expectedDateOfDelivery;
    }
}
