import * as d3 from 'd3';
import {AfterViewInit, Component, ElementRef, HostListener, Input, ViewChild} from '@angular/core';
import {Observable} from 'rxjs';
import {
  ObservationTable,
  ObservationTableMotherRows,
  ObservationTableRows
} from '@nspop/gm-web-facade-api';

@Component({
  selector: 'app-clinical-exams-line-chart',
  templateUrl: './clinical-exams-line-chart.component.html',
  styleUrls: ['./clinical-exams-line-chart.component.scss']
})
export class ClinicalExamsLineChartComponent implements AfterViewInit {
  // @ts-ignore
  @ViewChild('container') container: ElementRef;
  @Input() public data: Observable<ObservationTable | undefined> | undefined;
  public PRESET_HEIGHT = 400;
  public svg!: d3.Selection<SVGSVGElement, any, HTMLElement, undefined>;
  private width = 800;
  private height = this.PRESET_HEIGHT;
  private margin: { left: number; right: number; top: number; bottom: number; } = {top: 10, right: 30, bottom: 30, left: 60};
  private scaleY: any;
  private scaleX: any;
  public motherMeasurementRows: ObservationTableMotherRows[] | undefined;
  private tooltip: any;

  constructor() {
  }
  ngAfterViewInit(): void {
    if (!this.svg) {
      this.initializeChart();
      this.renderChart(this.width);
    }
    if (this.data) {
      this.data.subscribe( data => {
        if (data?.mother) {
          this.motherMeasurementRows = data.mother.rows?.filter(
            row => row.pubismeasurement !== null && row.weeksIntoPregnancy !== undefined);
          this.initData();
        }
      });
    }
  }

  @HostListener('window:resize', ['$event']) // for window scroll events
  public onResize($event: Event): void {
    this.width = this.container.nativeElement.offsetWidth;
    if (!this.svg) {
      this.initializeChart();
      this.renderChart(this.width);
    }
    if (this.motherMeasurementRows) {
      this.initData();
    }
  }

  public initData(): void {
    const mouseover = () => {
      this.tooltip
        .style('opacity', 1);
    };
    const mousemove = (event: MouseEvent, d: ObservationTableMotherRows) => {
      this.tooltip
        .html('<h4>Måling</h4><div>Symfysefundus mål: ' + d.pubismeasurement?.value + ' ' + d.pubismeasurement?.unit + '</div><div>Gestationsalder: ' + d.gestationAge + '</div>')
        .style('top', (event.pageY - 88) + 'px')
        .style('left', (event.pageX - 78) + 'px');
    };
    const mouseleave = () => {
      this.tooltip
        .transition()
        .duration(200)
        .style('opacity', 0);
    };
    if (this.motherMeasurementRows && this.motherMeasurementRows.length > 0 ) {
      const circle = this.svg.selectAll('circle')
        .data(this.motherMeasurementRows);
      circle.enter().append('circle')
        .attr('cy', (d) => {
          if (d.pubismeasurement?.value) {
            return this.scaleY(d.pubismeasurement.value / 10);
          } else {
            return 0;
          }
        })
        .attr('cx', (d) => this.scaleX(d.weeksIntoPregnancy))
        .attr('fill', (d) => '#4228C4')
        .attr('stroke', '#FFF')
        .attr('stroke-width', 3)
        .attr('filter', 'url(#dropshadow)')
        .attr('r', 6)
        .on('mouseover', mouseover )
        .on('mousemove', mousemove )
        .on('mouseleave', mouseleave );
    }
  }
  public renderChart(width: number): void {
    if (!this.container) {
      return;
    }
    this.width = width - this.margin.left - this.margin.right;
    this.height = this.PRESET_HEIGHT - this.margin.top - this.margin.bottom;
    this.svg.attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom)
      .append('g')
      .attr('transform',
        'translate(' + this.margin.left + ',' + this.margin.top + ')');
    this.createFilters();
    this.drawAxis();
    // this.drawAverageRange();
  }

  private drawAxis(): void {
    this.scaleX = d3.scaleLinear()
      .domain([4, 43])
      .range([0, this.width]);
    this.scaleY = d3.scaleLinear()
      .domain([0, 45])
      .range([this.height, 0]);
    this.svg.append('g')
      .attr('transform', 'translate(0,' + this.height + ')')
      .call(d3.axisBottom(this.scaleX));
    const yAxis = d3.axisLeft(this.scaleY)
      .ticks(7)
      .tickFormat((d: any, i: number) => i % 2 === 0 ? d : null);
    const yAxisGrid = d3
      .axisLeft(this.scaleY)
      .tickSize(-this.width)
      .ticks(7);
    this.svg.append('g')
      .attr('class', 'axis')
      .call(yAxisGrid);
    this.svg.append('g')
      .attr('class', 'axis')
      .call(yAxis);

    const gridColor = '#DDDDE4';
    this.svg.selectAll('line')
      .style('stroke', gridColor);
    this.svg.selectAll('path')
      .style('stroke', gridColor);
  }

  private createFilters(): void {
    /* For the drop shadow filter... */
    const defs = this.svg.append('defs');

    const filter = defs.append('filter')
      .attr('id', 'dropshadow');

    filter.append('feDropShadow')
      .attr('stdDeviation', 0.3)
      .attr('dx', 0.3)
      .attr('dy', 0.5);

    const feMerge = filter.append('feMerge');

    feMerge.append('feMergeNode')
      .attr('in', 'offsetBlur');
    feMerge.append('feMergeNode')
      .attr('in', 'SourceGraphic');
  }

  private initializeChart(): void {
    if (this.svg) {
      this.svg.remove();
    }
    this.svg = d3.select<SVGElement, any>(this.container.nativeElement).append('svg');
    this.width = this.width - this.margin.left - this.margin.right;
    this.height = this.PRESET_HEIGHT - this.margin.top - this.margin.bottom;
    this.svg
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom);
    this.scaleX = d3.scaleLinear()
      .domain([16, 42])
      .range([0, this.width]);
    this.scaleY = d3.scaleLinear()
      .domain([0, 45])
      .range([this.height, 0]);
    this.tooltip = d3.select('body')
      .append('div')
      .style('z-index', '500')
      .style('opacity', 0)
      .attr('class', 'tooltip')
      .style('border', 'solid')
      .style('border-width', '1px')
      .style('border-radius', '20px')
      .style('padding', '10px');
  }

  // TODO: Reimplement this when we have legitimate ranges
  private isWithinRange(point: ObservationTableRows): boolean {
    function getRangeFromWeek(week: number): { lower: number; upper: number } {
      return {lower: week - 6, upper: week + 6};
    }

    if (point.weeksIntoPregnancy != null && point.fetusWeightEstimate?.value != null) {
      const rangeFromWeek = getRangeFromWeek(point.weeksIntoPregnancy);
      return point.fetusWeightEstimate.value >= rangeFromWeek.lower && point.fetusWeightEstimate.value <= rangeFromWeek.upper;
    }
    return false;
  }
}
