import {TestBed} from '@angular/core/testing';

import {ActivePathwayGuard} from './active-pathway.guard';
import {ActivatedRouteSnapshot, Router, RouterStateSnapshot} from '@angular/router';
import {PatientContextService} from '../services/patient-context.service';
import SpyObj = jasmine.SpyObj;
import {createPatientContextSpyObject} from '../services/patient-context.service.spec';
import createSpyObj = jasmine.createSpyObj;
import {appUrls} from '../../utils/URL_STRING_LITERALS';

describe('ActivePathwayGuard', () => {
  let guard: ActivePathwayGuard;
  let routerMock: SpyObj<Router>;
  let activatedRouteStub: ActivatedRouteSnapshot;

  let patientContextServiceMock: SpyObj<PatientContextService>;

  beforeEach(async () => {
    routerMock = createSpyObj<Router>('Router', ['navigate']);

    activatedRouteStub = {} as any;

    patientContextServiceMock = createPatientContextSpyObject();

    await TestBed.configureTestingModule({
      imports: [],
      providers: [
        {provide: Router, useValue: routerMock},
        {provide: PatientContextService, useValue: patientContextServiceMock},
      ],
    }).compileComponents();
    guard = TestBed.inject(ActivePathwayGuard);
  });

  function fakeRouterState(url: string): RouterStateSnapshot {
    return {
      url,
    } as RouterStateSnapshot;
  }

  it('should be created', () => {
    expect(guard).toBeTruthy();
  });

  describe('when it has an active pathway', () => {
    beforeEach(() => {
      spyOn(guard, 'hasActivePathway').and.returnValue(true);
    });

    it('activates', () => {
      const cantActivate = guard.canActivate(activatedRouteStub, fakeRouterState(''));
      expect(cantActivate).toBe(true);
    });
  });

  describe('when it does not have an active pathway', () => {
    beforeEach(() => {
      spyOn(guard, 'hasActivePathway').and.returnValue(false);
    });

    it('does not activate', () => {
      const cantActivate = guard.canActivate(activatedRouteStub, fakeRouterState(''));
      expect(cantActivate).toBe(false);
    });

    describe('and the state url is already noActivePathway', () => {
      const routerState = fakeRouterState(appUrls.noActivePathway);
      it('does not navigate to noActivePathway path', () => {
        guard.canActivate(activatedRouteStub, routerState);
        expect(routerMock.navigate).toHaveBeenCalledTimes(0);
      });
    });

    describe('and the state url is NOT already noActivePathway', () => {
      const routerState = fakeRouterState('');
      it('navigates to noActivePathway path', () => {
        guard.canActivate(activatedRouteStub, routerState);
        expect(routerMock.navigate).toHaveBeenCalledTimes(1);
        expect(routerMock.navigate).toHaveBeenCalledWith([appUrls.noActivePathway]);
      });
    });
  });

  describe('hasActivePathway', () => {
    describe('when episode of care is set', () => {
      beforeEach(() => {
        patientContextServiceMock.getCurrentEpisodeOfCare.and.returnValue('1234eoc');
      });

      it('returns true', () => {
        expect(guard.hasActivePathway()).toBe(true);
      });
    });
    describe('when episode of care is NOT set', () => {
      beforeEach(() => {
        patientContextServiceMock.getCurrentEpisodeOfCare.and.returnValue(undefined);
      });

      it('returns false', () => {
        expect(guard.hasActivePathway()).toBe(false);
      });
    });
  });
});
