import {TestBed} from '@angular/core/testing';

import {PatientInContextGuard} from './patient-in-context.guard';
import {ActivatedRouteSnapshot, Router, RouterStateSnapshot} from '@angular/router';
import {PatientContextService} from './services/patient-context.service';
import SpyObj = jasmine.SpyObj;
import createSpyObj = jasmine.createSpyObj;
import {createPatientContextSpyObject} from './services/patient-context.service.spec';
import {appUrls} from '../utils/URL_STRING_LITERALS';

describe('PatientInContextGuard', () => {
  let guard: PatientInContextGuard;
  let routerMock: SpyObj<Router>;
  let activatedRouteStub: ActivatedRouteSnapshot;

  let patientContextServiceMock: SpyObj<PatientContextService>;

  beforeEach(async () => {
    routerMock = createSpyObj<Router>('Router', ['navigate']);

    activatedRouteStub = {} as any;

    patientContextServiceMock = createPatientContextSpyObject();

    await TestBed.configureTestingModule({
      imports: [],
      providers: [
        {provide: Router, useValue: routerMock},
        {provide: PatientContextService, useValue: patientContextServiceMock},
      ],
    }).compileComponents();
    guard = TestBed.inject(PatientInContextGuard);
  });

  function fakeRouterState(url: string): RouterStateSnapshot {
    return {
      url,
    } as RouterStateSnapshot;
  }

  it('should be created', () => {
    expect(guard).toBeTruthy();
  });

  describe('when it has a patient in context', () => {
    beforeEach(() => {
      spyOn(guard, 'hasPatientInContext').and.returnValue(true);
    });

    it('activates', () => {
      const cantActivate = guard.canActivate(activatedRouteStub, fakeRouterState(''));
      expect(cantActivate).toBe(true);
    });
  });

  describe('when it does not have a patient in context', () => {
    beforeEach(() => {
      spyOn(guard, 'hasPatientInContext').and.returnValue(false);
    });

    it('does not activate', () => {
      const cantActivate = guard.canActivate(activatedRouteStub, fakeRouterState(''));
      expect(cantActivate).toBe(false);
    });

    describe('and the state url is already an error route', () => {
      const routerStates = [appUrls.noPatientChosen, appUrls.somethingWentWrong, appUrls.cprDoesNotExist]
        .map(route => fakeRouterState(route));
      routerStates.forEach(routerState => it('does not navigate to noPatientChosen path', () => {
        guard.canActivate(activatedRouteStub, routerState);
        expect(routerMock.navigate).toHaveBeenCalledTimes(0);
      }));
    });

    describe('and the state url is NOT already an error route', () => {
      const routerState = fakeRouterState('');
      it('navigates to noPatientChosen path', () => {
        guard.canActivate(activatedRouteStub, routerState);
        expect(routerMock.navigate).toHaveBeenCalledTimes(1);
        expect(routerMock.navigate).toHaveBeenCalledWith([appUrls.noPatientChosen]);
      });
    });
  });

  describe('hasPatientInContext', () => {
    describe('when cpr is set', () => {
      beforeEach(() => {
        patientContextServiceMock.getCurrentCpr.and.returnValue('1234cpr');
      });
      describe('and patient exists', () => {
        beforeEach(() => {
          patientContextServiceMock.getCurrentPatientExists.and.returnValue(true);
        });

        it('returns true', () => {
          expect(guard.hasPatientInContext()).toBe(true);
        });
      });

      describe('and patient does not exist', () => {
        beforeEach(() => {
          patientContextServiceMock.getCurrentPatientExists.and.returnValue(false);
        });

        it('returns false', () => {
          expect(guard.hasPatientInContext()).toBe(false);
        });
      });
    });
  });

  describe('when cpr is NOT set', () => {
    beforeEach(() => {
      patientContextServiceMock.getCurrentCpr.and.returnValue(undefined);
    });

    it('returns false', () => {
      expect(guard.hasPatientInContext()).toBe(false);
    });
  });
});
