import {Component, forwardRef} from '@angular/core';
import {
  DataCard, PreviousAbortionsSection,
  PreviousAbortionsSectionAllOfAbortions, PreviousBirthsSection,
  PreviousBirthsSectionAllOfBirths,
  PreviousBirthsSectionAllOfChildren
} from '@nspop/gm-web-facade-api';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
// @ts-ignore
import deepEqual from 'deep-equal';
import {JsonTransformerUtil} from '../../../utils/json-transformer.util';
import {DataCardService} from '../../api/data-card.service';


export const APP_PREVIOUS_PREGNANCIES_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => PreviousPregnanciesComponent),
  multi: true
};

@Component({
  selector: 'app-previous-pregnancies',
  templateUrl: './previous-pregnancies.component.html',
  styleUrls: ['./previous-pregnancies.component.scss'],
  providers: [APP_PREVIOUS_PREGNANCIES_CONTROL_VALUE_ACCESSOR],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class PreviousPregnanciesComponent implements ControlValueAccessor {
  public disabled = false;
  public yes = true;
  public no = false;
  public abortions: AbortionBundle[] = [];
  public births: BirthBundle[] = [];

  private innerModel: DataCard | undefined;
  private dirtyAble = false;
  private onChangeCallback: (newValue: DataCard|undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor(public dataCardService: DataCardService) {
    this.dataCardService.isDataCardClosed.subscribe(isClosed => this.setDisabledState(isClosed));
  }

  get value(): DataCard|undefined {
    return this.innerModel;
  }

  set value(value: DataCard|undefined) {
    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      this.abortions = this.innerModel?.previousAbortionsSection?.abortions?.map(
        (abortion, index) => new AbortionBundle(abortion, index)
      ) || [];
      this.births = this.innerModel?.previousBirthsSection?.births?.map(
        (birth, index) => new BirthBundle(birth, index)
      ) || [];
      if (this.dirtyAble) {
        this.onChangeCallback(value);
        this.onTouchedCallback();
      }
      if (!this.dirtyAble && this.innerModel) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  writeValue(newValue: DataCard): void {
    this.value = newValue;
  }

  public removeAbortion(item: AbortionBundle): PreviousAbortionsSection | undefined {
    const card = this.copyCard();
    // tslint:disable-next-line:no-non-null-assertion if we call this we have an abortion to remove and hence things should not be null-ish
    card.previousAbortionsSection!.abortions!.splice(item.index, 1);
    // tslint:disable-next-line:no-non-null-assertion
    card.previousAbortionsSection!.dirty = true;
    this.value = card;
    return card.previousAbortionsSection;
  }

  public removeBirth(item: BirthBundle): PreviousBirthsSection | undefined {
    const card = this.copyCard();
    // tslint:disable-next-line:no-non-null-assertion  -- if we call this we have a birth to remove and hence things should not be null-ish
    card.previousBirthsSection!.births!.splice(item.index, 1);
    // tslint:disable-next-line:no-non-null-assertion
    card.previousBirthsSection!.dirty = true;
    this.value = card;
    return card.previousBirthsSection;
  }

  public addFetus(item: BirthBundle): PreviousBirthsSectionAllOfBirths {
    const fetus: PreviousBirthsSectionAllOfChildren = {
      childsCurrentHealthStatus: '',
      sex: undefined,
      outcome: undefined,
      birthWeight: {
        value: undefined as unknown as number,
        unit: 'g'
      }
    };
    item.birth.children?.push(fetus);
    return item.birth;
  }

  public removeFetus(item: BirthBundle, fetusIndex: number): PreviousBirthsSectionAllOfBirths | undefined {
    if (this.value?.previousBirthsSection) {
      item.birth.children?.splice(fetusIndex, 1);
      this.value.previousBirthsSection.dirty = true;
      return item.birth;
    } else {
      return undefined;
    }
  }

  public addAbortion(): PreviousAbortionsSectionAllOfAbortions {
    const abortion: PreviousAbortionsSectionAllOfAbortions = {
      year: undefined,
      type: undefined,
      gestationLength: {
        value: undefined as unknown as number,
        unit: 'w'
      },
      indication: ''
    };
    const card = this.copyCard();
    // tslint:disable-next-line:no-non-null-assertion  -- if we call this we have a birth to remove and hence things should not be null-ish
    if (card.previousAbortionsSection?.abortions) {
      card.previousAbortionsSection.abortions?.unshift(abortion);
    }else {
      const abortions = [];
      abortions.unshift(abortion);
      card.previousAbortionsSection = {abortions};
    }
    card.previousAbortionsSection.dirty = true;
    this.value = card;
    return abortion;
  }

  public addBirth(): PreviousBirthsSectionAllOfBirths {
    const birth = {
      year: undefined,
      gestationLength: {
        value: undefined,
        unit: 'd'
      },
      birthPlace: '',
      pregnancySummary: '',
      birthSummary: '',
      subjectiveBirthExperience: undefined,
      puerperalSepsis: undefined,
      children: [
        {
          childsCurrentHealthStatus: '',
          sex: undefined,
          outcome: undefined,
          birthWeight: {
            value: undefined,
            unit: 'g'
          }
        }
      ]
    } as unknown as PreviousBirthsSectionAllOfBirths;

    const card = this.copyCard();
    if (card.previousBirthsSection) {
      card.previousBirthsSection.births?.unshift(birth);
    } else {
      const births = [];
      births.unshift(birth);
      card.previousBirthsSection = {
        births
      };
    }

    card.previousBirthsSection.dirty = true;
    this.value = card;

    return birth;
  }

  /**
   * Makes a deep copy of the data-card.
   * This is needed so we can mutate a copy of the card in order to hook into the natural change detection flow
   */
  private copyCard(): DataCard {
    return JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(this.value)));
  }

  public weightChanged($event: any, fetus: PreviousBirthsSectionAllOfChildren): void {
    if (fetus.birthWeight) {
      fetus.birthWeight.value = $event;
    } else {
      fetus.birthWeight = {value: $event};
    }
  }

  public gestationLengthChanged($event: any, abortion: PreviousAbortionsSectionAllOfAbortions): void {
    if (abortion.gestationLength) {
      abortion.gestationLength.value = $event;
    } else {
      abortion.gestationLength = {value: $event};
    }
  }
}

export class AbortionBundle {
  constructor(public abortion: PreviousAbortionsSectionAllOfAbortions, public readonly index: number) {}
}

export class BirthBundle {

  private innerGestationAge: string | undefined;

  constructor(public birth: PreviousBirthsSectionAllOfBirths, public readonly index: number) {
    this.updateInnerGestationAge();
  }

  get gestationAge(): string | undefined {
    return this.innerGestationAge;
  }

  set gestationAge(newValue: string | undefined) {
    if (!newValue) {
      this.birth.gestationLength = {unit: 'd', value: undefined};
      return;
    }

    const parts = newValue.split('+');
    const weeks = Number(parts?.[0]);
    const days = Number(parts?.[1] || 0);
    if (!this.birth.gestationLength) {
      this.birth.gestationLength = {unit: 'd', value: 0};
    }
    if (weeks != null &&  !isNaN(weeks)) {
      this.birth.gestationLength.value = weeks * 7 + days;
    } else {
      this.birth.gestationLength.value = 0;
    }
  }

  updateInnerGestationAge(): void {
    if (this.birth.gestationLength?.value === undefined) {
      return this.innerGestationAge = undefined;
    }

    const weeks = Math.floor((this.birth.gestationLength?.value || 0) / 7.0);
    const days = (this.birth.gestationLength?.value || 0) % 7.0;
    this.innerGestationAge = `${weeks}+${days}`;
  }
}
