import { ComponentFixture, TestBed } from '@angular/core/testing';

import { RangeInputComponent } from './range-input.component';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {FormsModule, NgForm, ReactiveFormsModule} from '@angular/forms';

describe('RangeInputComponent', () => {
  let component: RangeInputComponent;
  let fixture: ComponentFixture<RangeInputComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ RangeInputComponent ],
      imports: [AngularMaterialModule, NoopAnimationsModule, FormsModule, ReactiveFormsModule],
      providers: [
        {provide: NgForm, useValue: new NgForm([], [])}
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(RangeInputComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('changes focus to max input box when "/" is pressed', () => {
    const min = component.minInput?.nativeElement as HTMLInputElement;
    const max = component.maxInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(max as HTMLOrSVGElement, 'focus');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '0');
    emulateKeystroke(min, '/');

    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('changes focus to max input box when "-" is pressed', () => {
    const min = component.minInput?.nativeElement as HTMLInputElement;
    const max = component.maxInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(max as HTMLOrSVGElement, 'focus');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '0');
    emulateKeystroke(min, '-');

    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('changes focus to max input box when ":" is pressed', () => {
    const min = component.minInput?.nativeElement as HTMLInputElement;
    const max = component.maxInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(max as HTMLOrSVGElement, 'focus');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '0');
    emulateKeystroke(min, ':');

    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('changes focus to max input box when "ArrowRight" is pressed and min input is non-zero length', () => {
    const min = component.minInput?.nativeElement as HTMLInputElement;
    const max = component.maxInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(max as HTMLOrSVGElement, 'focus');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, 'ArrowRight');

    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('changes focus when max length reached in min input box', () => {
    component.maxLength = 2;
    fixture.detectChanges();
    const min = component.minInput?.nativeElement as HTMLInputElement;
    const max = component.maxInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(max as HTMLOrSVGElement, 'focus');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '0');
    emulateKeystroke(min, '0');

    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('changes focus to min input box when "Backspace" is pressed in maxInput and length = zero', () => {
    const min = component.minInput?.nativeElement as HTMLInputElement;
    const max = component.maxInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(min as HTMLOrSVGElement, 'focus');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '-');

    emulateKeystroke(max, 'Backspace');

    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('changes focus to min input box when "ArrowLeft" is pressed in maxInput and length = zero', () => {
    const min = component.minInput?.nativeElement as HTMLInputElement;
    const max = component.maxInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(min as HTMLOrSVGElement, 'focus');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '1');
    emulateKeystroke(min, '-');

    emulateKeystroke(max, 'ArrowLeft');

    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('returns undefined if both min and max is empty', () => {
    const onChange = jasmine.createSpy('onChange');
    component.type = 'number';
    component.writeValue({min: 5, max: 4});
    component.registerOnChange(onChange);
    component.handleMinFocus();
    fixture.detectChanges();
    component.writeValue({min: 5, max: undefined});
    fixture.detectChanges();
    expect(onChange).toHaveBeenCalledWith({min: 5, max: undefined});
    component.writeValue({min: undefined, max: undefined});
    expect(onChange).toHaveBeenCalledWith({min: undefined, max: undefined});

  });

  it('updates inner value when ngModel is updated on the outside', () => {
    component.handleMinFocus();
    component.writeValue({min: 5, max: 4});
    expect(component.min).toEqual(5);
    expect(component.max).toEqual(4);
  });

  it('does not register a change on the first value set', () => {
    // This is to avoid issues where values initialized from an async call will mark the form as dirty
    const mock = {
      onChange: (whatever: any) => undefined
    };
    const onChange = spyOn(mock, 'onChange');
    component.registerOnChange(onChange);
    component.writeValue({min: 5, max: 4});
    expect(onChange).not.toHaveBeenCalled();
    component.handleMinFocus();
    component.writeValue({min: 5, max: 5});
    expect(onChange).toHaveBeenCalled();
  });
});

function emulateKeystroke(input: HTMLInputElement, char: string): void {
  if ( char !== 'Backspace' && char !== 'ArrowLeft') {
    input.value += char;
  }
  input.dispatchEvent(new KeyboardEvent('keydown', {key: char}));
}
