import {Component, OnDestroy} from '@angular/core';
import {UserAuthorizationType, UserContextService} from '../services/user-context.service';
import {SorItem, SorService} from '../services/sor.service';
import {FormControl, Validators} from '@angular/forms';
import {Observable, of, Subscription} from 'rxjs';
import {debounceTime, distinctUntilChanged, startWith, switchMap} from 'rxjs/operators';
import {DialogService} from '../modals/dialog/dialog.service';

@Component({
  selector: 'app-role-selector',
  templateUrl: './role-selector.component.html',
  styleUrls: ['./role-selector.component.scss']
})
export class RoleSelectorComponent implements OnDestroy{
  public organizationFormControl = new FormControl('', Validators.required);
  public roleFormControl = new FormControl('', Validators.required);
  private selectedRoleSubscription: Subscription;
  private selectedOrganizationSorSubscription: Subscription;

  public lookUp$: Observable<SorItem[] | null> = this.organizationFormControl.valueChanges.pipe(
    startWith(''),
    // Emit after user stopped typing for x ms
    debounceTime(400),
    distinctUntilChanged((a: string, b: string) => a === b),
    // use switch map so as to cancel previous subscribed events, before creating new once
    switchMap((value: string) => {
      // Minimum input
      if (value.length > 2) {
        return this.sorService.lookUp(value.toLowerCase());
      } else {
        return of(null);
      }
    })
  );

  constructor(
    public sorService: SorService,
    public userContextService: UserContextService,
    private dialogService: DialogService
  ) {
    this.selectedRoleSubscription = userContextService.selectedRole$.subscribe(
    role => this.roleFormControl.setValue(role.educationType)
    );
    this.selectedOrganizationSorSubscription =  userContextService.selectedOrganizationSor$.subscribe(
    sor => this.organizationFormControl.setValue(sor.name)
    );
  }

  selectSor(sorItem: SorItem): void {
    this.userContextService.selectOrganizationSor(sorItem);
    this.closeIfDialog();
  }

  selectRole(role: UserAuthorizationType): void {
    this.userContextService.selectRole(role);
    this.closeIfDialog();
  }

  ngOnDestroy(): void {
    this.selectedOrganizationSorSubscription.unsubscribe();
    this.selectedOrganizationSorSubscription.unsubscribe();
  }

  private closeIfDialog(): void {
    try {
      this.dialogService.close(this); // FIXME: We should probably make a dedicated "i'm done"-button instead of closing on change...
    } catch (e) {
      // Since this component is both used in the login screen and in a dialog we may not always have a dialog to close
      // - we handle that here but ignoring the "you forgot to open the dialog first"-exception
      if (e.message !== 'Could not close dialog - did you try to close it without opening it?' ) {
        throw e;
      }
    }
  }
}
