import {TestBed} from '@angular/core/testing';

import {SessionStoragePatientContext, SessionStorageService, SessionStorageUserContext} from './session-storage.service';
import {WindowService} from './window.service';
import SpyObj = jasmine.SpyObj;
import createSpyObj = jasmine.createSpyObj;

describe('SessionStorageService', () => {
  let service: SessionStorageService;

  let windowServiceMock: SpyObj<WindowService>;

  beforeEach(() => {

    windowServiceMock = createSpyObj<WindowService>(
      'WindowService',
      {
        nativeWindow: () => ({
          document: {cookie: ''},
          sessionStorage: {
            getItem: () => {},
            setItem: () => {},
            removeItem: () => {},
            clear: () => {}
          }
        })
      }
    );

    TestBed.configureTestingModule({
      providers: [{provider: WindowService, useValue: windowServiceMock}]
    });
    service = TestBed.inject(SessionStorageService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('when setting user context to an empty object', () => {
    const context: SessionStorageUserContext = {};
    beforeEach(() => {
      spyOn(service, 'getUserContext').and.returnValue(context);
    });

    it('deletes the userContext', () => {
      const spy = spyOn(service, 'deleteUserContext');
      service.setUserContext(context);
      expect(spy).toHaveBeenCalledTimes(1);
    });
  });

  describe('when setting user context to a non-empty object', () => {
    const role = 'doctor' as any;
    const context: SessionStorageUserContext = { role };
    beforeEach(() => {
      spyOn(service, 'getUserContext').and.returnValue(context);
    });

    it('does not delete the userContext', () => {
      const spy = spyOn(service, 'deleteUserContext');
      service.setUserContext(context);
      expect(spy).toHaveBeenCalledTimes(0);
    });

    it('overwrites role when setting it', () => {
      const spy = spyOn(service, 'setUserContext');
      const roleNew = 'læge' as any;
      service.setUserContextRole(roleNew);
      expect(spy).toHaveBeenCalledTimes(1);
      expect(spy).toHaveBeenCalledWith({role: roleNew});
      expect(spy).not.toHaveBeenCalledWith(role);
    });

    it('adds userIdentifier to context when setting it', () => {
      const spy = spyOn(service, 'setUserContext');
      const userIdentifier = 12345;
      service.setUserIdentifier(userIdentifier);
      expect(spy).toHaveBeenCalledTimes(1);
      expect(spy).toHaveBeenCalledWith({role, userIdentifier});
    });

    it('adds organizationSor to context when setting it', () => {
      const spy = spyOn(service, 'setUserContext');
      const organizationSor = 12345 as any;
      service.setUserContextOrganization(organizationSor);
      expect(spy).toHaveBeenCalledTimes(1);
      expect(spy).toHaveBeenCalledWith({role, organizationSor});
    });

    it('adds commonName to context when setting it', () => {
      const spy = spyOn(service, 'setUserContext');
      const commonName = 'Jens' as any;
      service.setUserContextCommonName(commonName);
      expect(spy).toHaveBeenCalledTimes(1);
      expect(spy).toHaveBeenCalledWith({role, commonName});
    });
  });

  describe('when setting patient context to an empty object', () => {
    const context: SessionStoragePatientContext = {};
    beforeEach(() => {
      spyOn(service, 'getPatientContext').and.returnValue(context);
    });

    it('deletes the patientContext', () => {
      const spy = spyOn(service, 'deletePatientContext');
      service.setPatientContext(context);
      expect(spy).toHaveBeenCalledTimes(1);
    });
  });

  describe('when setting patient context to a non-empty object (cpr)', () => {
    const cpr = 'cpr1234';
    const context: SessionStoragePatientContext = {cpr} as any;
    beforeEach(() => {
      spyOn(service, 'getPatientContext').and.returnValue(context);
    });

    it('does not delete the patientContext', () => {
      const spy = spyOn(service, 'deletePatientContext');
      service.setPatientContext(context);
      expect(spy).toHaveBeenCalledTimes(0);
    });

    it('adds xBreakTheGlassReason to context when setting it', () => {
      const spy = spyOn(service, 'setPatientContext');
      const xBreakTheGlassReason = 'consent';
      service.setPatientContextXBreakTheGlassReason(xBreakTheGlassReason);
      expect(spy).toHaveBeenCalledTimes(1);
      expect(spy).toHaveBeenCalledWith({cpr, xBreakTheGlassReason});
    });

    it('overwrites cpr in context when setting it', () => {
      const spy = spyOn(service, 'setPatientContext');
      const cprNew = 'cpr4321';
      service.setPatientContextCpr(cprNew);
      expect(spy).toHaveBeenCalledTimes(1);
      expect(spy).toHaveBeenCalledWith({cpr: cprNew});
    });
  });
});
