import {Injectable, OnDestroy} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {environment} from '../../environments/environment';
import {delay, map, shareReplay, tap} from 'rxjs/operators';
import {combineLatest, Observable, Subject, Subscription} from 'rxjs';
import {filterNullish} from '../../utils/rxjs-operators';
import {SorItem} from './sor.service';
import {PatientContextService} from './patient-context.service';
import {SessionStorageService} from './session-storage.service';
import deepEqual from 'deep-equal';
import {WindowService} from './window.service';
import {CookieService} from 'ngx-cookie-service';

export interface UserContext {
  commonName: string;
  organizationName: string;
  eligibleRoles: UserAuthorizationType[];
  patientIdentifier?: string;
  organizationSor?: SorItem;
  userIdentifier: number;
}

export interface UserAuthorizationType {
  authorizationCode: string;
  educationCode: string;
  educationType: string;
}

export interface KeepAliveContext {
  configuredIdleTimeout: number;
}

@Injectable({
  providedIn: 'root'
})
export class UserContextService implements OnDestroy {

  private USER_CONTEXT_PATH = '/usercontext';
  private NEW_PATIENT_PATH = '/patientIdentifier';
  private LOGOUT_PATH = '/saml/Logout';
  private fetchUserContextSessionStorage: Subscription;
  private contextInitialized = false;
  private roleSelected = false;
  private orgSelected = false;

  constructor(private http: HttpClient,
              private patientContext: PatientContextService,
              private userContextHolder: UserContextHolder,
              private sessionStorageService: SessionStorageService,
              private cookieService: CookieService,
              private windowService: WindowService
  ) {
    // Subscribe to make the observable hot so that any previous context will be loaded
    this.fetchUserContextSessionStorage = this.userContext$.pipe(
      map(() => sessionStorageService.getUserContext())
    ).subscribe(
      sessionStorageContext => {
        if (sessionStorageContext?.commonName) {
          this.setCommonName(sessionStorageContext.commonName);
        }
        if (sessionStorageContext?.organizationSor) {
          this.selectOrganizationSor(sessionStorageContext.organizationSor);
        }
        if (sessionStorageContext?.role) {
          this.selectRole(sessionStorageContext.role);
        }
        if (sessionStorageContext?.userIdentifier) {
          this.setUserIdentifier(sessionStorageContext.userIdentifier);
        }

      }
    );

    combineLatest([
      this.userContext$,
      this.selectedRole$,
      this.selectedOrganizationSor$
    ]).pipe(
      filterNullish(),
      map(() => true),
      tap((initialized) => this.contextInitialized = initialized)
    ).subscribe();
  }

  public keepAlive$: Observable<KeepAliveContext> =
    this.http.get<KeepAliveContext>(environment.base_path + '/api' + this.USER_CONTEXT_PATH + '/keepAlive');

  public userContext$: Observable<UserContext> = this.http.get<UserContext>(environment.base_path + '/api' + this.USER_CONTEXT_PATH).pipe(
    tap(async (userContext: UserContext) => {
      const sessionStorageUserContext = this.sessionStorageService.getUserContext();
      const isSessionStorageUserEmpty = deepEqual({}, sessionStorageUserContext, {strict: true});

      if (isSessionStorageUserEmpty) {
        // Set session storage if empty
        const {commonName, organizationSor, userIdentifier, eligibleRoles} = userContext;
        this.sessionStorageService.setUserContextCommonName(commonName);
        this.sessionStorageService.setUserContextOrganization(organizationSor);
        this.sessionStorageService.setUserContextRole(eligibleRoles.length === 1 ? eligibleRoles[0] : undefined);
        this.sessionStorageService.setUserIdentifier(userIdentifier);
      }

      if (userContext.patientIdentifier) {
        // Open new patient if SBO login and none is in session context (i.e. in a new session)
        const {cpr} = this.sessionStorageService.getPatientContext();
        if (!cpr) {
          await this.openPatient(userContext.patientIdentifier);
        }
      }

      this.patientContext.loadPatientContextFromSessionStorage();
    }),
    delay(environment.mock ? 3000 : 0),
    shareReplay(),
  );

  public eligibleRoles$: Observable<UserAuthorizationType[]> =
    this.userContext$.pipe(
      tap(userContext => {
        if (userContext.eligibleRoles && userContext.eligibleRoles.length === 1) {
          this.selectRole(userContext.eligibleRoles[0]);
        }
      }),
      map(userContext => userContext.eligibleRoles),
      shareReplay()
    );

  private innerSelectedRole: Subject<UserAuthorizationType> = new Subject();
  public selectedRole$: Observable<UserAuthorizationType> = this.innerSelectedRole
    .pipe(
      tap(selectedRole => this.sessionStorageService.setUserContextRole(selectedRole)),
      tap(() => this.roleSelected = true),
      shareReplay()
    );

  private innerSelectedOrganizationSor: Subject<SorItem> = new Subject();
  public selectedOrganizationSor$: Observable<SorItem> = this.innerSelectedOrganizationSor
    .pipe(
      tap(selectedSor => console.log('selected organizationSor:' + selectedSor)),
      tap(selectedSor => this.sessionStorageService.setUserContextOrganization(selectedSor)),
      tap(() => this.orgSelected = true),
      shareReplay()
    );

  public async openPatient(cpr: string): Promise<void> {
    return new Promise((resolve, reject) => {
      this.http.put<void>(`${environment.base_path}/api${this.USER_CONTEXT_PATH}${this.NEW_PATIENT_PATH}/${cpr}`, undefined)
        .subscribe(() => {
            this.patientContext.setCpr(cpr);
            resolve();
          },
          (e) => reject(e));
    });
  }

  public logout(): void {
    this.patientContext.setCpr();
    this.sessionStorageService.clear();
    this.cookieService.deleteAll();
    this.windowService.nativeWindow().location.assign(environment.base_path + this.LOGOUT_PATH);
  }

  public isContextInitialized(): boolean {
    return this.contextInitialized;
  }

  public isRoleAndOrgSelected(): boolean {
    return this.roleSelected && this.orgSelected;
  }

  public keepAlive(): void {
    this.keepAlive$.subscribe();
  }

  public setCommonName(userName: string): void {
    this.userContextHolder._setName(userName);
    this.sessionStorageService.setUserContextCommonName(userName);
  }

  public selectRole(role: UserAuthorizationType): void {
    console.log('UserContextService selectRole() : ' + JSON.stringify(role));
    this.userContextHolder._setCurrentRole(role);
    this.innerSelectedRole.next(role);
  }

  private setUserIdentifier(userIdentifier?: number): void {
    this.userContextHolder._setUniqueIdenfier(userIdentifier);
    this.sessionStorageService.setUserIdentifier(userIdentifier);
  }

  public selectOrganizationSor(organizationSor: SorItem): void {
    console.log('UserContextService selectOrganizationSor() : ' + JSON.stringify(organizationSor));
    this.userContextHolder._setCurrentOrganization(organizationSor);
    this.innerSelectedOrganizationSor.next(organizationSor);
  }

  ngOnDestroy(): void {
    this.fetchUserContextSessionStorage.unsubscribe();
  }

}

@Injectable({
  providedIn: 'root'
})
export class UserContextHolder {
  private selectedName?: string;
  private selectedOrganization?: SorItem;
  private selectedRole?: UserAuthorizationType;
  private userIdentifier?: number;

  constructor() {
  }

  get commonName(): string | undefined {
    return this.selectedName;
  }

  get currentOrganization(): SorItem | undefined {
    return this.selectedOrganization;
  }

  get currentRole(): UserAuthorizationType | undefined {
    return this.selectedRole;
  }

  get uniqueIdentifier(): number | undefined {
    return this.userIdentifier;
  }

  public _setName(commonName: string): void {
    this.selectedName = commonName;
  }

  public _setCurrentOrganization(org: SorItem | undefined): void {
    this.selectedOrganization = org;
  }

  public _setCurrentRole(role: UserAuthorizationType | undefined): void {
    this.selectedRole = role;
  }

  public _setUniqueIdenfier(identifier: number | undefined): void {
    this.userIdentifier = identifier;
  }
}
