import {
  AfterViewInit,
  Component,
  ContentChildren,
  ElementRef,
  forwardRef,
  Input,
  OnChanges,
  QueryList,
  SimpleChanges,
  ViewChild
} from '@angular/core';
import {fromEvent, Observable, of} from 'rxjs';
import {SorItem, SorService} from '../services/sor.service';
import {debounceTime, distinctUntilChanged, filter, finalize, first, map, tap} from 'rxjs/operators';
import {DataCardService} from '../api/data-card.service';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
import {MatOption} from '@angular/material/core';
import deepEqual from 'deep-equal';
import {MatSelect} from '@angular/material/select';
import {HealthCareOrganization} from '@nspop/gm-web-facade-api';

export type SorLookupType = string | HealthCareOrganization | SorItem;

export const APP_SOR_LOOKUP_INPUT_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => SorLookupInputComponent),
  multi: true
};

@Component({
  selector: 'app-sor-lookup-input',
  templateUrl: './sor-lookup-input.component.html',
  styleUrls: ['./sor-lookup-input.component.scss'],
  providers: [APP_SOR_LOOKUP_INPUT_CONTROL_VALUE_ACCESSOR],
  // tslint:disable-next-line:no-host-metadata-property
  host: {
    '(mouseenter)': 'onMouseEnter()',
    '(mouseleave)': 'onMouseLeave()',
    '(blur)': 'onBlur()',
  }
})

export class SorLookupInputComponent implements ControlValueAccessor, OnChanges, AfterViewInit {

  constructor(private elementRef: ElementRef,
              private sorService: SorService,
              private dataCardService: DataCardService) {
    this.dataCardService.isDataCardClosed.subscribe(isClosed => this.setDisabledState(isClosed));
  }

  @Input() label = '';
  @Input() name = '';
  @Input() notRelevant = false;
  @Input() debounceInMs: number | undefined = 500;
  @Input() type: 'name' | 'healthCareOrganization' | 'sorItem' = 'name';
  @Input() allowFreeText: boolean = false; // Only works in conjunction with type = 'name'

  public autocompleteOptions: SorLookupType[] = [];

  @ViewChild('input', {static: true}) input: ElementRef | undefined;
  @ViewChild('select', {static: true}) select: MatSelect | undefined;
  @ContentChildren(MatOption, {descendants: true}) options: QueryList<MatOption>|undefined;

  public innerModel: string | SorItem | undefined;
  public disabled = false;
  public hover = false;
  public required = false;

  public sorServiceIsDoingLookup: boolean = false;
  public lookupValue: string = '';
  public sorLookupSubscription = of().subscribe();

  // When notRelevant is toggled on, we store the old value, so we can re-fill it it gets turned off again
  private oldValue: any;
  private dirtyAble = false;

  private onChangeCallback: (newValue: string) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  public sorAutocompleteOptions$: (lookupValue: string) => Observable<SorLookupType[]> =
    (lookupValue: string) => {
      if (lookupValue.length < 2) { return of([] as SorLookupType[]); }

      const sorItemsObservable = this.sorService.lookUp(lookupValue.toLowerCase());

      switch (this.type) {
        case 'name': {
          return sorItemsObservable.pipe(map(sorItems => sorItems.map(
            (sorItem: SorItem) => sorItem.name))
          );
        }
        case 'healthCareOrganization': {
          return sorItemsObservable.pipe(map(sorItems => sorItems.map(
            (sorItem: SorItem) => ({name: sorItem.name, sorId: sorItem.sorId} as HealthCareOrganization)))
          );
        }
      }

      return sorItemsObservable;
    }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.notRelevant) {
      if (this.notRelevant) {
        this.oldValue = this.innerModel;
        this.writeValue(undefined);
      } else {
        this.writeValue(this.oldValue);
      }
    }
  }

  ngAfterViewInit(): void {
    if (this.input) {
      // We handle debounce this slightly complicated way to avoid change detection running for every debounced keystroke
      fromEvent<KeyboardEvent>(this.input.nativeElement, 'keyup')
        .pipe(
          filter<KeyboardEvent>((event =>
            event.key !== 'Enter'
            && event.key !== 'ArrowUp'
            && event.key !== 'ArrowDown' // We don't make another search if any of these keys are pressed.
          )),
          debounceTime(this.debounceInMs || 0),
          // tslint:disable-next-line:no-non-null-assertion We literally just checked if it was null/undefined...
          distinctUntilChanged<KeyboardEvent>(() => this.value === this.input!.nativeElement.value),
          tap(() => {
            // tslint:disable-next-line:no-non-null-assertion We literally just checked if it was null/undefined...
            this.lookupValue = this.input!.nativeElement.value;
            if (this.isTypeEqualName() && this.allowFreeText) {
              this.writeValue(this.lookupValue);
            }
          })
        )
        .subscribe(
          () => {
            // first() pipe unsubscribes after first value is received, so we don't 'over-subscribe'.
            this.sorServiceIsDoingLookup = true;
            this.sorLookupSubscription = this.sorAutocompleteOptions$(this.lookupValue)
              .pipe(
                first(),
                finalize(() => {
                  this.sorServiceIsDoingLookup = false;
                  }))
              .subscribe((options: SorLookupType[]) => {
              this.autocompleteOptions = options;
            });
          }
        );
    }
    this.required = this.elementRef.nativeElement.hasAttribute('required') &&
      this.elementRef.nativeElement.attributes.required.value !== 'false';
  }

  onMouseEnter(): void {
    this.hover = !(this.disabled || this.notRelevant);
  }

  onMouseLeave(): void {
    this.hover = false;
  }

  // Set touched on blur
  onBlur(): void {
    this.onTouchedCallback();
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  get value(): any {
    return this.innerModel;
  }

  set value(value: any) {
    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      this.setLookupValue(value);
      if (this.dirtyAble) {
        this.onChangeCallback(value);
      }
    }
  }

  writeValue(newValue: any): void {
    this.value = newValue;
  }

  makeDirtyable(): void { // on user input we're naturally dirty-able
    this.dirtyAble = true;
  }

  handleSelectOpen(): void {
    this.focusAutocompleteInput();
  }

  handleSelectClose(): void {
    if (this.lookupValue === '') {
      this.writeValue(undefined);
    } else {
      this.setLookupValue(this.value);
    }
  }

  handleCloseButtonClick(): void {
    this.select?.close();
  }

  focusAutocompleteInput(): void {
    (this.input?.nativeElement as HTMLInputElement).select();
  }

  preventKeyEventPropagation(event: KeyboardEvent): void {
    // Disable selection with space key - This way we are able to type space in the input without closing the selection menu.
    event.stopPropagation();
  }

  getSelectDisplayValue(): string | undefined {
    if (!this.value) { return; }
    return this.isTypeEqualName() ? this.value : this.value.name;
  }

  setLookupValue(value?: SorLookupType): void {
    if (!value) {
      this.lookupValue = '';
      return;
    }
    this.lookupValue = this.isTypeEqualName() ? (value as string) : (value as HealthCareOrganization | SorItem).name;
  }

  isTypeEqualName(): boolean {
    return this.type === 'name';
  }
}
