import {HttpErrorResponse, HttpHeaders, HttpInterceptor, HttpRequest, HttpResponse} from '@angular/common/http';
import {HttpRequestInterceptor} from './interceptor';
import {ActiveRequests} from '../app/services/active-requests.service';
import {of, throwError} from 'rxjs';
import {fakeAsync, flush, TestBed} from '@angular/core/testing';
import {UserContextHolder} from '../app/services/user-context.service';
import {PatientContextService} from '../app/services/patient-context.service';
import {AppError, ErrorService} from '../app/services/error.service';
import {HttpClientTestingModule} from '@angular/common/http/testing';

describe('Http Interceptor', () => {
  let service: HttpInterceptor;
  let activeRequests: ActiveRequests;
  let userContext: UserContextHolder;
  let patientContext: PatientContextService;
  let errorServiceStub: ErrorService;

  beforeEach(async () => {
    activeRequests = {
      _registerBlockingRequest: () => undefined,
      _unregisterBlockingRequest: () => undefined,
    } as any;

    userContext = {
      get currentOrganization(): any {
        return {
          name: 'name',
          sorId: 'sorId',
          institutionOwnerSorId: 'institutionOwnerSorId'
        };
      },
      get currentRole(): any {
        return {
          authorizationCode: 'authorizationCode',
          educationCode: 'educationCode',
          educationType: 'educationType'
        };
      }
    } as any;

    patientContext = {
      getCurrentEpisodeOfCare: () => 'episodeOfCareId',
      getCurrentCpr: () => 'testCpr',
      getCurrentXBreakTheGlassReason: () => 'stangspring',
    } as any;

    errorServiceStub = {
      handleHttpError: (error: any) => {
        throw new AppError(error);
      },
      handleHttpErrorResponse: (error: any) => {
        return throwError(error);
      },
      redirectIfRequested: () => {},
    } as any;

    await TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        HttpRequestInterceptor,
        {provide: ActiveRequests, useValue: activeRequests},
        {provide: UserContextHolder, useValue: userContext},
        {provide: PatientContextService, useValue: patientContext},
        {provide: ErrorService, useValue: errorServiceStub},
      ]
    }).compileComponents();
    service = TestBed.inject(HttpRequestInterceptor);
  });

  it('adds headers from context', () => {
    let headers = {} as HttpHeaders;
    service.intercept(
      new HttpRequest<any>('GET', '/whatever'),
      {handle: (requestWithHeaders) => {
            headers = requestWithHeaders.headers;
            return of(new HttpResponse({body: {}, status: 200, statusText: 'OK'}));
        }}
    );
    expect(headers.get('X-NSP-GM-Chosen-Role')).toEqual('educationType');
    expect(headers.get('X-NSP-GM-Organization-Sor')).toEqual('sorId');
    expect(headers.get('X-NSP-GM-Institution-Owner-Sor')).toEqual('institutionOwnerSorId');
    expect(headers.get('X-NSP-GM-Patient-Identifier')).toEqual('testCpr');
    expect(headers.get('X-NSP-GM-Episode-Of-Care-Identifier')).toEqual('episodeOfCareId');
    expect(headers.get('X-NSP-GM-Break-The-Glass-Reason')).toEqual('stangspring');
  });

  it('does not add headers if their value would be undefined/empty', () => {
    patientContext.getCurrentEpisodeOfCare = () => undefined;
    patientContext.getCurrentCpr = () => '';
    patientContext.getCurrentXBreakTheGlassReason = () => undefined;
    let headers = {} as HttpHeaders;
    service.intercept(
      new HttpRequest<any>('GET', '/whatever'),
      {handle: (requestWithHeaders) => {
          headers = requestWithHeaders.headers;
          return of(new HttpResponse({body: {}, status: 200, statusText: 'OK'}));
        }}
    );
    expect(headers.get('X-NSP-GM-Patient-Identifier')).toBeFalsy();
    expect(headers.get('X-NSP-GM-Episode-Of-Care-Identifier')).toBeFalsy();
    expect(headers.get('X-NSP-GM-Chosen-Role')).toEqual('educationType');
    expect(headers.get('X-NSP-GM-Organization-Sor')).toEqual('sorId');
    expect(headers.get('X-NSP-GM-Institution-Owner-Sor')).toEqual('institutionOwnerSorId');
    expect(headers.get('X-NSP-GM-Break-The-Glass-Reason')).toBeFalsy();
  });

  it('increments active requests when intercepting', () => {
    const activeRequestService = spyOn(activeRequests, '_registerBlockingRequest').and.callThrough();
    service.intercept(
      new HttpRequest<any>('GET', '/whatever'),
      {handle: () => of(new HttpResponse({body: {}, status: 200, statusText: 'OK'}))}
    );
    expect(activeRequestService).toHaveBeenCalledTimes(1);
  });

  it('decrements active requests when  request finishes', fakeAsync(() => {
    const unregisterBlockingRequest = spyOn(activeRequests, '_unregisterBlockingRequest').and.callThrough();
    service.intercept(
      new HttpRequest<any>('GET', '/whatever'),
      {handle: () => of(new HttpResponse({body: {}, status: 200, statusText: 'OK'}))}
    ).subscribe(() => undefined);
    flush();
    expect(unregisterBlockingRequest).toHaveBeenCalledTimes(1);
  }));

  it('decrements active requests even if a request errors', fakeAsync(() => {
    const error = new HttpErrorResponse({error: 'oops', status: 500 });
    const unregisterBlockingRequest = spyOn(activeRequests, '_unregisterBlockingRequest').and.callThrough();
    service.intercept(new HttpRequest<any>('GET', '/whatever'), {handle: () => throwError(error)})
      .subscribe(() => undefined);
    expect(() => flush()).toThrow(error);
    expect(unregisterBlockingRequest).toHaveBeenCalledTimes(1);
  }));

  it('transforms date strings into js Date-objects', (done) => {
    service.intercept(
      new HttpRequest<any>('GET', '/whatever'),
      {handle: () => of(new HttpResponse({body: {whateverDate: '2020-11-08T11:06:15.31'}, status: 200, statusText: 'OK'}))}
    ).subscribe((event) => {
      expect(typeof (event as any).body.whateverDate).toBe('object');
      expect((event as any).body.whateverDate.toDateString()).toEqual('Sun Nov 08 2020');
      done();
    });
  });

});
